package de.cnc.expression;

import de.cnc.expression.exceptions.ExpressionEvaluationException;

/**
 * This contents the variables, localization and other runtime informations
 *
 * Spter soll hier die Umgebung des CnC, also der JSP-Adressraum, request,
 * session und application eingesetzt werden.
 *
 * Denkbar ist folgende Schreibweise:
 * session.user, request.form, application.config
 */
abstract public class AbstractRuntimeEnvironment
{
    /**
     * Variable lesen
     * 
     * @param paStrName Name der Variable
     * @return Wert der Variable
     */
   abstract public Object getVariable(String paStrName)
       throws ExpressionEvaluationException;
   
   /**
    * Variable schreiben
    * 
    * @param paStrName
    * @param paObjValue
    * @throws ExpressionEvaluationException
    */
   abstract public void setVariable(final String pVarName, final Object pVarValue)
     throws ExpressionEvaluationException;
   
    /**
     * Variable lschen
     * 
     * @param paStrName
     * @return the removed value
     */
    abstract public Object removeVariable(String paStrName)
        throws ExpressionEvaluationException;

    // --- Typ-Prfung ---
    /**
     * Rckgabe Variablen-Typ
     * 
     * @param paStrName
     * @return the removed value
     */
    abstract public Class setVariableType(//--
            final String pVarName, //--
            final Class pVarClass)
        throws ExpressionEvaluationException;

    /**
     * Rckgabe Variablen-Typ
     * 
     * @param paStrName
     * @return the removed value
     */
    abstract public Class getVariableType(//--
            final String pVarName)
        throws ExpressionEvaluationException;
    

    // --- Sicherheitsschalter fr Member-Zugriff ---

    protected boolean memberEnabled = true;

    /**
     * @return Returns the memberEnabled.
     */
    public boolean isMemberEnabled()
    {
        return this.memberEnabled;
    }
    /**
     * @param memberEnabled The memberEnabled to set.
     */
    public void setMemberEnabled(boolean memberEnabled)
    {
        this.memberEnabled = memberEnabled;
    }

    // --- Sicherheitsschalter fr Methoden-Aufruf ---

    protected boolean invokeEnabled = true;

    /**
     * @return
     */
    public boolean isInvokeEnabled()
    {
        return this.invokeEnabled;
    }

    /**
     * @param invokeEnabled The invokeEnabled to set.
     */
    public void setInvokeEnabled(boolean invokeEnabled)
    {
        this.invokeEnabled = invokeEnabled;
    }

}
