package de.cnc.expression;

// import de.cnc.expression.*;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;

public abstract class AbstractToken
{

    protected int iLine;
    protected int iCol;
    protected String strSource;
    protected int iSrcLength = 0;
    protected String strOriginalSource;

    // public AbstractToken(String paStr, int paIntLine, int paIntCol) {
    // strSource = paStr;
    public AbstractToken(
        String paStrOriginalSource,
        int paIntLine,
        int paIntCol)
    {
        this.strOriginalSource = paStrOriginalSource;
        this.strSource = paStrOriginalSource; // hier mal ordentlich nacharbeiten
        this.iLine = paIntLine;
        this.iCol = paIntCol;
    }

    /**
     * gets the the source line number
     */
    public final int getLine()
    {
        return this.iLine;
    }

    /**
     * gets the col position in the source code line
     */
    public final int getCol()
    {
        return this.iCol;
    }

    public final String getSource()
    {
        return this.strSource;
    }

    public final int getSrcLength()
    {
        return this.iSrcLength;
    }

    public final String getOriginalSource()
    {
        return this.strOriginalSource;
    }

    /**
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    // abstract public final Object[] getCheckReturnTypes(AbstractRuntimeEnvironment pRunEnv) ;
    abstract public Object[] getCheckReturnTypes(final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionParseException;

    /**
     * rekursives Aufrufen der Klammerung der Ausdrcke
     * entsprechend der Operator-Prioritt.
     */
    abstract public void organizeOperatorPriority();

    /**
     * process the token
     */
    abstract public Object eval(final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionEvaluationException;

    /**
     * process the token
     */
    // abstract public  void checkTypes(AbstractRuntimeEnvironment pRunEnv) throws ExpressionParseException;

    /**
     * Prfen ob es sich um einen konstanten Wert
     * (Literal oder konstanter Ausdruck handelt)
     */
    abstract public boolean isConst(final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionEvaluationException;

    /**
     * Zurckgeben des konstanten Wertes
     * oder null wenn es keinen konstanten
     * Wert gibt.
     */
    abstract public Object getConstVal(final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionEvaluationException;

    /**
     * @param i
     */
    public void setCol(final int i)
    
    {
        this.iCol = i;
    }

    /**
     * @param i
     */
    public void setLine(final int i)
    {
        this.iLine = i;
    }

} // end class
