package de.cnc.expression;


import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.expression.infixoperators.AbstractInfixOperator;
import de.cnc.expression.util.AbstractInfixOperatorArrayList;
import de.cnc.expression.util.AbstractTokenArrayList;
import de.cnc.expression.util.SubExpressionArrayList;


/**
 * Diese Klasse sammelt alle Operanden und Operatoren eines
 * gleichen Operator-Priority-Levels
 * 
 * @author Heiner Kcker
 * @since 27.05.2004
 */
public class OperatorPriorityLevel extends 
//SubExpression
AbstractToken
{
    
    /** die Prioritt dieses Levels numerisch */
    private int opLevelInt;

    //protected SubExpressionArrayList subExpressionList = new SubExpressionArrayList();
    protected AbstractTokenArrayList subExpressionList = new AbstractTokenArrayList();

    protected AbstractInfixOperatorArrayList infixOperatorList = new AbstractInfixOperatorArrayList();

    /**
     * constructor
     */
    
    /* package*/ OperatorPriorityLevel(
            int pOpLevelInt
            
            /* String paStr, int paIntLine, int paIntCol*/
            )
    {
        // super(paStr, paIntLine, paIntCol);
        super("", 0, 0);
        this.opLevelInt = pOpLevelInt;
    }// end constructor

    /**
     * process the token
     */
    public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
    {
//System.out.println("this: " + this);

        /*SubExpression*/ AbstractToken subExp = this.subExpressionList.get(0);

        Object retObject = subExp.eval(pRunEnv);

        // Abarbeitung der Liste mit den SubExpressions und den Operatoren
        for (int i = 1; i < this.subExpressionList.size(); i++)
        {
            AbstractInfixOperator infixOperator = this.infixOperatorList.get(i - 1); // i- 1 es gibt immer einen Operator weniger als SubExpressions

            subExp = this.subExpressionList.get(i);
            retObject = infixOperator.eval(retObject, subExp, pRunEnv);
        }// for

        return retObject;
    }// end method eval

    public /*SubExpression*/ AbstractToken getFirstSubExpression()
    {
        if (this.subExpressionList != null && this.subExpressionList.size() > 0)
        {
            return this.subExpressionList.get(0);
        }
        return null;
    }


    
    public String toString()
    {
        // mal umbauen auf StringBuffer
        String strRet = " <<" + getClass().getName() + "\nopLevelInt: " + this.opLevelInt;

        if (this.subExpressionList.size() > 0)
        {
            strRet += "\n0;" + this.subExpressionList.get(0);
            // Abarbeitung der Liste mit den SubExpressions und den Operatoren
            for (int i = 1; i < this.subExpressionList.size(); i++)
            {
                strRet += "\n" + i + ":" + this.infixOperatorList.get(i - 1);
                strRet += "\n" + i + ":" + this.subExpressionList.get(i);
            }// for
        }
        strRet += ">> ";
        return strRet;
    }// end method toString

    /* package*/ static OperatorPriorityLevel organizeOperatorPriority(
            final SubExpressionArrayList pSubExpressionList,
            final AbstractInfixOperatorArrayList pInfixOperatorList)
    {
        // der niedrigste Level ist 1
        final OperatorPriorityLevel opLevel = new OperatorPriorityLevel(1);

        // Start rekursiver Durchlauf
        organizeOperatorPriorityRecursiv(opLevel, new Position(0), pSubExpressionList,
                pInfixOperatorList);

        // int a = 1; //stop for debugger

        // System.out.println("opLevel: ======================================\n "+opLevel);

        return opLevel;
    }

    /**
     * rekursives Organisieren der Operator-Priority . <br>
     * 
     * Beim Auftreten einer hheren Operator-Prioritt als der aktuellen
     * wird eine Klammer-Ebene eingeschoben.
     * 
     * @param pStartInt Start-Position in der SubExpressionList
     * @return aktuelle Position in der SubExpressionList
     */
    private static void organizeOperatorPriorityRecursiv(
            final OperatorPriorityLevel pOpLevel,
            Position pRunPos,
            final SubExpressionArrayList pSubExpressionList,
            final AbstractInfixOperatorArrayList pInfixOperatorList)
    {
//System.out.println("als Parameter bergeben:\n"+pOpLevel);

        // Abarbeitung der Liste mit den SubExpressions und den Operatoren
        for (; pRunPos.pos < pSubExpressionList.size(); pRunPos.pos++)
        {
            // System.out.println("pRunPos: "+pRunPos);
            final SubExpression subExp = pSubExpressionList.get(pRunPos.pos);

            // System.out.println("subExp: "+subExp);

            subExp.organizeOperatorPriority();

            if (pRunPos.pos >= pInfixOperatorList.size())
            {
                // letzter Operand erreicht, kein Operator mehr da
                pOpLevel.subExpressionList.add(subExp);
            }
            else
            {
                AbstractInfixOperator infixOperator = pInfixOperatorList.get(pRunPos.pos);

                // System.out.println("infixOperator: "+infixOperator);

                if (infixOperator.getPriorityLevel() < pOpLevel.opLevelInt)
                {
                    // Level ist gesunken
                    // System.out.println( "============== Level ist gesunken ====================");
                    pOpLevel.subExpressionList.add(subExp);
                    return; 
                }
                else if (infixOperator.getPriorityLevel() > pOpLevel.opLevelInt)
                {
                    // System.out.println( "============== Level ist gestiegen ====================");

                    // Level ist gestiegen
                    OperatorPriorityLevel opLevel = new OperatorPriorityLevel(pOpLevel.opLevelInt
                            + 1);

                    organizeOperatorPriorityRecursiv(opLevel, pRunPos, pSubExpressionList,
                            pInfixOperatorList);

                    //int a = 1; // stop for debugger

                    // System.out.println("opLevel return: ---------------------- \n"+opLevel);

                    pOpLevel.subExpressionList.add(opLevel);

                    if (pRunPos.pos < pInfixOperatorList.size())
                    {
                        // letzter Operand noch nicht erreicht, aktuellen Operator aufaddieren
                        infixOperator = pInfixOperatorList.get(pRunPos.pos);
                        pOpLevel.infixOperatorList.add(infixOperator);
                    }
                }
                else
                {
                    // System.out.println( "============== gleichbleibendes Level ====================");
                    // gleichbleibendes Level
                    pOpLevel.subExpressionList.add(subExp);
                    pOpLevel.infixOperatorList.add(infixOperator);
                }
            }
            // subExp = pSubExpressionList.get(i);
        }// for
        // System.out.println( "============== Zhler abgelaufen ====================");
    }

    /**
     * Kopie von Expression#getCheckReturnTypes
     * 
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    public final Object[] getCheckReturnTypes(//--
            final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionParseException
    {

      /*SubExpression*/ AbstractToken subExp = this.subExpressionList.get(0);

      Object[] retTypes = subExp.getCheckReturnTypes(pRunEnv);

      // Abarbeitung der Liste mit den SubExpressions und den Operatoren
      for (int i = 1; i < this.subExpressionList.size(); i++) {
        AbstractInfixOperator infixOperator = this.infixOperatorList.get(i - 1); // i- 1 es gibt immer einen Operator weniger als SubExpressions

        subExp = this.subExpressionList.get(i);
        Object[] rightTypes = subExp.getCheckReturnTypes(pRunEnv);

        retTypes = infixOperator.getCheckReturnTypes(retTypes, rightTypes);
      }// for

      return retTypes;

    }// end method getCheckReturnTypes

    /**
     * @see de.cnc.expression.AbstractToken#organizeOperatorPriority()
     */
    public final void organizeOperatorPriority()
    {
        throw new RuntimeException("darf hier nicht aufgerufen werden");
    }

    /**
     * Diese Klasse dient als Hilfslsung
     * fr die fehlende Mglichkeit zweier
     * Returnwerte fr eine Methode in Java.
     */
    public static class Holder
    {
        public SubExpressionArrayList subExpressionList;

        public AbstractInfixOperatorArrayList infixOperatorList;
    }


    public static class Position
    {
        public int pos;

        public Position(int pPos)
        {
            this.pos = pPos;
        }
    
        public String toString()
        {
            return this.getClass().getName() + " pos: " + this.pos;
        }
    }


    /* (non-Javadoc)
     * @see de.cnc.expression.AbstractToken#isConst(de.cnc.expression.AbstractRuntimeEnvironment)
     */
    public boolean isConst(AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
    {
        return this.subExpressionList.isConst(pRunEnv);
    }

    /* (non-Javadoc)
     * @see de.cnc.expression.AbstractToken#getConstVal(de.cnc.expression.AbstractRuntimeEnvironment)
     */
    public Object getConstVal(AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
    {
        return eval(pRunEnv);
    }

}// end class
