package de.cnc.expression;


import java.lang.reflect.Array;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.tokensimple.ReservedWord;


/**
 * This class contents the variables, localization and other runtime informations
 * 
 * Hier werden die Variablen fr die allein arbeitenden Version der
 * Expression-Library vorgehalten.
 */
public class StandaloneRuntimeEnvironment
    extends AbstractRuntimeEnvironment
{

    /** HashMap mit den Variablenwerten */
    private HashMap varMap = new HashMap();

    /**
     * Variable lesen
     * 
     * @param paStrName Name der Variable
     * @return Wert der Variable
     */
   public Object getVariable(String paStrName)
   {
      return this.varMap.get(paStrName);
   }

    /**
     * Variable schreiben
     * 
     * @param pVarName
     * @param pVarValue
     * @throws ExpressionEvaluationException
     */
   public void setVariable(//--
           final String pVarName, //--
           final Object pVarValue)
      throws ExpressionEvaluationException
   {
      if (!isVariableNameValid(pVarName))
      {
         throw new ExpressionEvaluationException(0, 0, "", "",
               "variable name " + pVarName + " is invalid");
      }
      if (ReservedWord.isReservedWord(pVarName))
      {
         throw new ExpressionEvaluationException(0, 0, "", "",
               "variable name " + pVarName + " is an reserved word");
      }
      
      //Typ-Prfung
      Class varType = getVariableType(pVarName);
      if ( varType != null //--
              && pVarValue != null //--
              && ! varType.isInstance( pVarValue ) )
      {
          throw new ExpressionEvaluationException(0, 0, "", "",
                  "variable type " + varType.getName() + " expected instead " + pVarValue.getClass().getName() );
      }
      this.varMap.put(pVarName, pVarValue);
   }

    /**
     * Variable lschen
     * @param paStrName
     * @return the removed value
     */
   public Object removeVariable(String paStrName)
   {
      return this.varMap.remove(paStrName);
   }

    /**
     * Debug-Ausgabe
     */
   public String toString()
   {
      return getClass().getName() + " " + this.varMap;
   }

   // first char an letter, rest chars letter or digit
   // maximal length Variable.VAR_NAME_MAX_LEN
   private static boolean isVariableNameValid(String paStrName)
   {
//      if (paStrName.length() > Variable.VAR_NAME_MAX_LEN)
//      {
//         return false;
//      }
      if (paStrName.length() > 0 && Character.isJavaIdentifierStart(paStrName.charAt(0)))
      {
         for (int i = 1; i < paStrName.length(); i++)
         {
            char cTmp = paStrName.charAt(i);
            if ( cTmp == ':' // Doppelpunkt im Variablen-Namen erlaubt
                 /*&& i < Variable.VAR_NAME_MAX_LEN*/
                 && i < paStrName.length() - 1
                 && Character.isJavaIdentifierStart(paStrName.charAt( i + 1 ) ) ) {
            }
            else if (!(Character.isJavaIdentifierPart(cTmp))) {
              return false;
            }
         }
         return true;
      }
      return false;
   }// end method

   /**
    * formatierte Ausgabe als HTML
    */
   public String toHtmlString()
   {
      StringBuffer sb = new StringBuffer();
      Set keySet = this.varMap.keySet();
      Iterator it = keySet.iterator();

      while (it.hasNext())
      {
         String strKey = (String) it.next();

         sb.append("<b>"+strKey + ": </b>");
         Object oValue = this.varMap.get(strKey);

         if (oValue != null && oValue.getClass().isArray())
         {
            // ein Array
            sb.append("Array length:" + Array.getLength(oValue) + " [");
            for (int i = 0; i < Array.getLength(oValue); i++)
            {
               sb.append(Array.get(oValue, i));
               if (i < Array.getLength(oValue)-1)
               {
                  sb.append(", ");
               }
            }
            sb.append("]");
         }
         else
         {
            sb.append(oValue);
         }
         sb.append("<br><br>");
      }
      return sb.toString();

   }// end method toHtmlString

   /** HashMap mit den Variablentypen */
   private HashMap typMap = new HashMap();

   /* (non-Javadoc)
    * @see de.cnc.expression.AbstractRuntimeEnvironment#setVariableType(java.lang.String, java.lang.Class)
    */
   public Class setVariableType(String pVarName, Class pVarClass)
       throws ExpressionEvaluationException
   {
//Log.debug( pVarName, pVarClass );
       //throw new UnsupportedOperationException();
       if ( pVarName == null )
       {
           throw new NullPointerException();
       }
       if ( pVarClass == null )
       {
           throw new NullPointerException();
       }
       this.typMap.put( pVarName , pVarClass );

       // Prfen, ob ein evtl. vorhandener Wert zum Typ passt
       Object value = getVariable(pVarName);
       if (value != null //--
               && ! pVarClass.isInstance( value ) )
       {
           throw new RuntimeException( "variable " + pVarName + " has wrong type " + value.getClass().getName() 
                   + " expected was " + pVarClass.getName() );
       }
       
       return pVarClass;
   }


   /* (non-Javadoc)
    * @see de.cnc.expression.AbstractRuntimeEnvironment#getVariableType(java.lang.String)
    */
   public Class getVariableType(String pVarName) throws ExpressionEvaluationException
   {
       //throw new UnsupportedOperationException();
       return (Class) this.typMap.get(pVarName);
   }


}// end class

