package de.cnc.expression.assignmentoperators;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Assignable;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


public abstract class AbstractAssignmentOperator {

  protected int iLine;
  protected int iCol;
  protected String strSource;
  protected int iSrcLength = 0;
  protected String strOriginalSource;

  /**
   * protected Konstruktor
   * instanziieren verboten
   */
  protected AbstractAssignmentOperator() {}

  /**
   * Konstruktor
   */
  public AbstractAssignmentOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    // System.out.println("constructor AbstractAssignmentOperator "+ paStrOriginalSource ) ;
    this.strOriginalSource = paStrOriginalSource;
    this.iLine = paIntLine;
    this.iCol = paIntCol;
  }

  /**
   * gets the the source line number
   */
  public int getLine() {
    return this.iLine;
  }

  /**
   * gets the col position in the source code line
   */
  public int getCol() {
    return this.iCol;
  }

  public String getSource() {
    return this.strSource;
  }

  public int getSrcLength() {
    return this.iSrcLength;
  }

  public String getOriginalSource() {
    return this.strOriginalSource;
  }

  /**
   * Prfroutine fr Variablen-Namen . <br>
   * @see de.cnc.expression.tokencomplex.Variable#parse
   */
  public static boolean isOperator(String strPa) {
    return
        ":=".equals(strPa) || // :=
        "+=".equals(strPa) || // +=
        "-=".equals(strPa) || // -=
        "*=".equals(strPa) || // *=
        "/=".equals(strPa) || // /=
        ":<=".equals(strPa) || // :<=
        ":>=".equals(strPa) || // :>=
        ":min=".equals(strPa) || // :min=
        ":max=".equals(strPa)  // :max=
        ;
  }// end method isOperator

  /**
   * Parse assignment operators
   *
   * @return geparsten Operator oder null wenn nichts passendes gefunden
   */
  public static AbstractAssignmentOperator parseAssignmentOperator(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException {
//DebugUtil.printMethodInfo();
//System.out.println( "parseAssignmentOperator paStr: >"+paStr+"<");

    // dirty trick
    // Ausnutzung verkrzte Auflsung logischer Ausdrcke

    AbstractAssignmentOperator retOperator = null;
    boolean bFake = // .
        ((retOperator = SetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // .
        ((retOperator = PlusSetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // .
        ((retOperator = MinusSetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // .
        ((retOperator = TimesSetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // .
        ((retOperator = DivideSetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // .
        ((retOperator = MinSetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // .
((retOperator = MaxSetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) ; //.

    return retOperator;

  }// end method parseInfixOperator

  /**
   * debug output
   */
  public String toString() {
    String strRet = getClass().getName() + " source: " + getSource();

    return strRet;
  }// end method toString

  /**
   * process the operator . <br>
   */
  abstract public Object eval(final Assignable pInxOp, final Object paObjLeft, final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException ;

  /**
   * rekursives Aufrufen der Klammerung der Ausdrcke
   * entsprechend der Operator-Prioritt.
   * Nichts zu tun.
   */
  void organizeOperatorPriority()
  {
      ;
  }
  
  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  abstract public Object[] getCheckReturnTypes(//--
          final Object[] pClassArrLeft, //--
          final Object[] pClassArrRight)
      throws ExpressionParseException;

  /**
   * vorgefertigte Typ-prfung auf Num-Parameter und Num-Rckgabewert
   */
  public static Object[] checkNumNumNum(//--
          final AbstractAssignmentOperator paAssOp, //--
          final Object[] paIntArrLeft, //--
          final Object[] paIntArrRight)
    throws ExpressionParseException {
    Object[] iTypesArr = paIntArrLeft;

    if ( ! Types.isInOrUnknown(Number.class, iTypesArr)) {
      throw new ExpressionParseException(paAssOp.getLine(), paAssOp.getCol(), paAssOp.getOriginalSource(),
          "numeric type expected");
    }
    iTypesArr = paIntArrRight;
    if ( ! Types.isInOrUnknown(Number.class, iTypesArr)) {
      throw new ExpressionParseException(paAssOp.getLine(), paAssOp.getCol(), paAssOp.getOriginalSource(),
          "numeric type expected");
    }

    //return new Object[]{ Number.class };
    return new Object[] { Number.class };
  }// end method getCheckReturnTypes

}// end class
