package de.cnc.expression.assignmentoperators;


import java.math.BigDecimal;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Assignable;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * Divide Set Operator <b>*=</b> divide and set
 */
public class DivideSetOperator extends AbstractAssignmentOperator {

  /**
   * parse
   */
  public static DivideSetOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { 

    if ("/=".equals(paStr) || (paStr.length() > 2 && paStr.startsWith("/=") && !Character.isLetterOrDigit(paStr.charAt(2)))) {
      DivideSetOperator dvdSetOperator = new DivideSetOperator(paStrOriginalSource, paIntLine, paIntCol);

      dvdSetOperator.strSource = "/=";
      dvdSetOperator.iSrcLength = 2;
      return dvdSetOperator;
    }
    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no || operator");
  }// end method parse

  /**
   * constructor
   */
  public DivideSetOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator . <br>
   */
  public final Object eval(final Assignable paVariable, Object paObjRight, AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Object objLeft = paVariable.eval(pRunEnv);

    Object objResult = innerEval(paObjRight, objLeft);
    paVariable.set(pRunEnv, objResult);

    return objResult;
  }// end method eval

 /**
 * @param paObjRight
 * @param pObjLeft
 * @return
 * @throws ExpressionEvaluationException
 */
private Object innerEval(Object paObjRight, Object pObjLeft) throws ExpressionEvaluationException
{
    if (pObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(pObjLeft instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + pObjLeft + ":" + pObjLeft.getClass().getName() + " ; BigDecimal expected");
    }

    if (!(paObjRight instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; BigDecimal expected");
    }
    if (paObjRight == null) {
      throw new ExpressionEvaluationException(this, "right operand is null");
    }

    final String strLeftValue = "" + pObjLeft;
    final String strRightValue = "" + paObjRight;

    // System.out.println("strLeftValue:" + strLeftValue);
    // System.out.println("strRightValue:" + strRightValue);

    Object objResult = null;

    // Abfangung division durch Null
    try {
      objResult = (new BigDecimal(strLeftValue)).setScale(6, BigDecimal.ROUND_HALF_EVEN).divide(new BigDecimal(strRightValue),
          BigDecimal.ROUND_HALF_EVEN); // ROUND_HALF_EVEN);
    } catch (Exception e) {
      throw new ExpressionEvaluationException(this, "" + e);
    }
    return objResult;
}

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Divide-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final Object[] pClassArrLeft, //--
          final Object[] pClassArrRight)
      throws ExpressionParseException 
  {
    return AbstractAssignmentOperator.checkNumNumNum(this,pClassArrLeft, pClassArrRight); // Rckgabe der zu setzenden Typen
  }// end method getCheckReturnTypes

}// end class
