package de.cnc.expression.assignmentoperators;


import java.math.BigDecimal;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Assignable;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * Minimum Set Operator <b>:>=</b> oder <b>:min=</b>
 */
public class MinSetOperator extends AbstractAssignmentOperator {

  /**
   * parse
   */
  public static MinSetOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException {

    if (":>=".equals(paStr) || (paStr.length() > 3 && paStr.startsWith(":>=") && !Character.isLetterOrDigit(paStr.charAt(3)))
       
        ) {
      MinSetOperator minSetOperator = new MinSetOperator(paStrOriginalSource, paIntLine, paIntCol);

      minSetOperator.strSource = ":>=";
      minSetOperator.iSrcLength = 3;
      return minSetOperator;
    }

    if (":min=".equals(paStr)
        || (paStr.length() > 5 && paStr.startsWith(":min=") && !Character.isLetterOrDigit(paStr.charAt(5)))
       
        ) {
      MinSetOperator minSetOperator = new MinSetOperator(paStrOriginalSource, paIntLine, paIntCol);

      minSetOperator.strSource = ":min=";
      minSetOperator.iSrcLength = 5;
      return minSetOperator;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no || operator");
  }// end method parse

  /**
   * constructor
   */
  public MinSetOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator . <br>
   */
  public final Object eval(final Assignable paVariable, final Object paObjRight, AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {

    Object objLeft = paVariable.eval(pRunEnv);

    if (objLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (objLeft instanceof Boolean) {
      // dirty trick, Umwandlung in String
      // weil t im Alphabet nach f folgt, ist true > false ( 0 > 1 )
      // klappt nur in englisch
      objLeft = "" + objLeft;
    }
    else    if (objLeft instanceof Number) {
      // Normieren numerische Werte auf BigDecimal
      objLeft = new BigDecimal("" + objLeft);
    }
    if (!(objLeft instanceof Comparable)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + objLeft + ":" + objLeft.getClass().getName() + " ; Comparable expected");
    }
    Comparable cmpLeftValue = (Comparable) objLeft;

    // Hilfsvariable, weil der rechte Wert je nach Typ angepasst werden muss
    Object objRight = paObjRight;

    if (objRight == null) {
      throw new ExpressionEvaluationException(this, "right operand is null");
    }
    if (objRight instanceof Boolean) {
      // dirty trick, Umwandlung in String
      // da t im Alphabet nach f folgt, ist true > false ( 0 > 1 )
      // klappt nur in englisch
      // in deutsch ( j n ) nicht
      objRight = "" + objRight;
    }
    else if (objRight instanceof Number) {
      // Normieren numerische Werte auf BigDecimal
      objRight = new BigDecimal("" + objRight);
    }
    if (!(objRight instanceof Comparable)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + objRight + ":" + objRight.getClass().getName() + " ; Comparable expected");
    }

    Comparable cmpRightValue = (Comparable) objRight;

    try {
      if (cmpLeftValue.compareTo(cmpRightValue) > 0) {
        paVariable.set(pRunEnv, paObjRight);

        return paObjRight;      	
      	
      }
      return objLeft;
      	
    } catch (Exception e) {
      throw new ExpressionEvaluationException(this, "" + e);
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(final Object[] paIntArrLeft, final Object[] paIntArrRight)  throws ExpressionParseException {
    return AbstractAssignmentOperator.checkNumNumNum(this,paIntArrLeft, paIntArrRight); // Rckgabe der zu setzenden Typen
  }// end method getCheckReturnTypes

}// end class
