package de.cnc.expression.assignmentoperators;


import java.math.BigDecimal;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Assignable;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * Add Set Operator <b>+=</b>
 */
public class PlusSetOperator extends AbstractAssignmentOperator {

  public static PlusSetOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {

    if ("+=".equals(paStr) || (paStr.length() > 2 && paStr.startsWith("+=") && !Character.isLetterOrDigit(paStr.charAt(2)))) {
      PlusSetOperator addSetOperator = new PlusSetOperator(paStrOriginalSource, paIntLine, paIntCol);

      addSetOperator.strSource = "+=";
      addSetOperator.iSrcLength = 2;
      return addSetOperator;
    }
    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no || operator");
  }// end method parse

  /**
   * constructor
   */
  public PlusSetOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator . <br>
   */
  public final Object eval(final Assignable paVariable, Object paObjRight, AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {

    // DebugUtil.printStackTrace();

    Object objLeft = paVariable.eval(pRunEnv);

    if (objLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (objLeft instanceof Number) {
      String strLeftValue = "" + objLeft;

      if (paObjRight == null) {
        throw new ExpressionEvaluationException(this, "right operand is null");
      }
      if (!(paObjRight instanceof Number)) {
        throw new ExpressionEvaluationException(this,
            "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; BigDecimal expected");
      }
      String strRightValue = "" + paObjRight;

      // System.out.println("strLeftValue:" + strLeftValue);
      // System.out.println("strRightValue:" + strRightValue);

      Object objResult = (new BigDecimal(strLeftValue)).add(new BigDecimal(strRightValue));

      // System.out.println("objResult:" + objResult);

      paVariable.set(pRunEnv, objResult);

      return objResult;
    }
    else if (objLeft instanceof String) {
      // Strings
      String strLeftValue = "" + objLeft;

      String strRightValue = "" + paObjRight;

      Object objResult = strLeftValue + strRightValue;

      paVariable.set(pRunEnv, objResult);
      return objResult;

    }
    else {
      throw new ExpressionEvaluationException(this,
          "left operand is " + objLeft + ":" + objLeft.getClass().getName() + " ; BigDecimal or String expected");
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(final Object[] paIntArrLeft, final Object[] paIntArrRight)  throws ExpressionParseException {
    return AbstractAssignmentOperator.checkNumNumNum(this,paIntArrLeft, paIntArrRight); // Rckgabe der zu setzenden Typen
  }// end method getCheckReturnTypes

}// end class
