package de.cnc.expression.exceptions;


import de.cnc.expression.assignmentoperators.AbstractAssignmentOperator;
import de.cnc.expression.infixoperators.AbstractInfixOperator;
import de.cnc.expression.prepostfixoperators.AbstractPrePostfixOperator;


public class ExpressionEvaluationException extends Exception {

  private int iLine;
  private int iCol;
  private String strSource;
  private String strOriginalSource;

  /**
   * constructor
   */
  public ExpressionEvaluationException(int paIntLine, int paIntCol, String paStrSource, String paStrOriginalSource, String paStrErrorMessage) {
    super(paStrErrorMessage);
    this.iLine = paIntLine;
    this.iCol = paIntCol;
    this.strSource = paStrSource;
    this.strOriginalSource = paStrOriginalSource;
  }

  /**
   * constructor
   */
  public ExpressionEvaluationException(AbstractInfixOperator paOp, String paStrMessage) {
    this(paOp.getLine(), paOp.getCol(), paOp.getSource(), paOp.getOriginalSource(), paStrMessage);
  }

  /**
   * constructor
   */
  public ExpressionEvaluationException(AbstractPrePostfixOperator paOp, String paStrMessage) {
    this(paOp.getLine(), paOp.getCol(), paOp.getSource(), paOp.getOriginalSource(), paStrMessage);
  }

  /**
   * constructor
   */
  public ExpressionEvaluationException(AbstractAssignmentOperator paOp, String paStrMessage) {
    this(paOp.getLine(), paOp.getCol(), paOp.getSource(), paOp.getOriginalSource(), paStrMessage);
  }

  public int getLine() {
    return this.iLine;
  }

  public int getCol() {
    return this.iCol;
  }

  public String getMessage() {
    return super.getMessage() + " line:" + this.iLine
        + " col:" + this.iCol + "\n"     // col
        // + strSource + "\n"       // source
        + this.strOriginalSource + "\n"  // original source entire line
        + spaces(this.iCol) + "^\n";   // advice arrow
  }

  private static String spaces(int paIntCount) {
    StringBuffer sb = new StringBuffer();

    for (int i = 0; i < paIntCount; i++) {
      sb.append(' ');
    }
    return sb.toString();
  }

}// end class ExpressionEvaluationException
