package de.cnc.expression.functions;

import java.util.List;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.util.Log;
import de.cnc.util.StringUtil;

/**
 * add( &lt;List or Array&gt; , &lt;value&gt; [ , &lt;index&gt; ] ) . <br>
 * Anfgen eines Wertes an eine {@link java.util.List} oder eine Array
 */
public class AddFunction extends AbstractFunction
{

   /**
    * constructor
    */
   public AddFunction(String paStrOriginalSource, int paIntLine, int paIntCol)
   {
      super(paStrOriginalSource, paIntLine, paIntCol);
   } // end constructor

   /**
    * process the token
    */
   public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
   {
      Object oListOrArray = this.paramList.get(0).eval(pRunEnv);
      Object addValue = this.paramList.get(1).eval(pRunEnv);
      Number listIndex = getNumberParamNullable(pRunEnv, 2);

      if (oListOrArray == null)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "first parameter missed");
      }
      if ( oListOrArray instanceof List )
      {
         List oList = (List) oListOrArray;
         if (listIndex == null)
         {
            //kein Index
            oList.add( addValue );
         }
         else
         {
            //Index-Angabe
            oList.add( listIndex.intValue() , addValue );
         }
         
         return oList;
      }
      if ( oListOrArray.getClass().isArray() )
      {
         Object[] oArr = (Object[]) oListOrArray;
         Object[] retArr = StringUtil.arrAdd( oArr , addValue );
         return retArr;
      }
      else
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "list or array expected");
      }

      //try
      //{
      //   Object oValue = paramList.get(1).eval(pRunEnv);

      //   return oValue;
      //}
      //catch (Exception e)
      //{
      //   throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
      //}

   } // end method eval

   /**
    * get the wahrscheinlichen return typen . <br>
    * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
    * null wenn kein Rckgabe-Wert,
    * mehrere wenn mehrere mglich
    * ( z.B. Plus-Operator kann numerisch oder String liefern)
    */
   public final Object[] getCheckReturnTypes(//--
           final AbstractRuntimeEnvironment pRunEnv)
       throws ExpressionParseException
   {
      checkParamCount(2 , 3);

      final Object[] typeArr = checkParameterForListOrArray(0,pRunEnv);

      if (this.paramList.size() > 2)
      {
         checkParameterForNumber(2,pRunEnv);
      }

      //return new int[] { Types.RET_TYPE_LIST_OR_ARRAY };
      if ( Types.isIn( List.class, typeArr) //--
              || Types.isIn( Object[].class, typeArr ) )
      {
          return typeArr;
      }
      if ( Types.contentsUnknown( typeArr ) )
      {
          //Warnung sollte eigentlich ber Log.warning laufen
          //TODO Log.debug( "ambigous parameter for " , this.getClass().getName() );
          return new Object[]{ List.class, Object[].class };
      }
      throw new ExpressionParseException(
              getLine(),
              getCol(),
              getOriginalSource(),
              "Object[] or java.util.List as first parameter expected");
      
   } // end method getCheckReturnTypes

} // end class
