package de.cnc.expression.functions;


import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * arrayToList( &lt;value&gt; ) . <br>
 * wandelt das bergebene Array in eine java.util.ArrayList um . <br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class ArrayToListFunction extends AbstractFunction 
{

  /**
   * constructor
   */
  public ArrayToListFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Object[] objArrValue = getArrayParamNullable( pRunEnv, 0 );

    if (objArrValue == null) {
      return null;
    }
    final List list = Arrays.asList( objArrValue );
    
    // conversion to java.util.List
    return new ArrayList( list );
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    checkParameterForArray(0, pRunEnv);
    
    return new Object[] { List.class , Types.NULL_CLASS };
    
    //return new int[] {Types.RET_TYPE_LIST_OR_ARRAY };
  }// end method getCheckReturnTypes

} // end class
