package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * assert( &lt;booleanExpresseion&gt; , &lt;Error-String&gt; ) . <br>
 * Ermittelt ob der per Parameter bergebene Ausdruck vom Typ Boolean 'true' ist.
 * Wenn nicht wird eine Exception mit dem bergebenen Fehlertext geworfen.
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 */
public class AssertFunction extends AbstractFunction {

  /**
   * constructor
   */
  public AssertFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Boolean boolValue = getBooleanParamNoNull(pRunEnv,0);
    if ( boolValue.equals( Boolean.FALSE ) )
    {
        String errTxt = getStringParamNoNull(pRunEnv, 1);
        throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), errTxt);
    }
    return Boolean.TRUE;
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(2);
    
    checkParameterForBoolean(0,pRunEnv);
    checkParameterForString(1,pRunEnv);

    //return new int[] {Types.RET_TYPE_BOOL };
    return new Object[]{ Boolean.class };
  }// end method getCheckReturnTypes

} // end class
