package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.BreakSequenceException;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * breakSequence( &lt;objectvalue&gt; ) . <br>
 * breakSequ    ( &lt;objectvalue&gt; ) . <br>
 * Bricht die aktuelle Sequenz ab und liefert den als Parameter bergebenen Wert zurck. <br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: BreakSequenceFunction.java,v 1.1 2005/04/12 10:09:15 hkuecker Exp $
 */
public class BreakSequenceFunction extends AbstractFunction {

  /**
   * constructor
   */
  public BreakSequenceFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Object objValue = null;

    if (this.paramList.size() > 0) {
      objValue = this.paramList.get(0).eval(pRunEnv);
    }
    // return objValue;
    throw new BreakSequenceException(objValue) ;
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(0, 1);
    
    //return new int[] {Types.RET_TYPE_UNKNOWN };
    return this.paramList.get(0).getCheckReturnTypes(pRunEnv);
  }// end method getCheckReturnTypes

} // end class
