package de.cnc.expression.functions;


import java.math.BigDecimal;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * ceil( &lt;valueNum&gt; ) . <br>
 * Aufrunden.<br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: CeilingFunction.java,v 1.1 2005/04/12 10:09:13 hkuecker Exp $
 */
public class CeilingFunction extends AbstractFunction {

  /**
   * constructor
   */
  public CeilingFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Number numValue = getNumberParamNoNull(pRunEnv, 0);

    try {
      double ceilValue = Math.ceil( numValue.doubleValue() ) ;
      return new BigDecimal(ceilValue) ;
    } catch (Exception e) {
       // Log.fatal( e );
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      //return new BigDecimal(0);
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    return checkParameterForNumber(0, pRunEnv);

    //return new Object[]{ Number.class };
  }// end method getCheckReturnTypes

} // end class
