package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Expression;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * compile( &lt;valueString&gt; ) . <br>
 * Kompiliert den bergebenen String
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: CompileFunction.java,v 1.1 2005/04/12 10:09:26 hkuecker Exp $
 */
public class CompileFunction extends AbstractFunction {

  /**
   * constructor
   */
  public CompileFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    String strValue = getStringParamNoNull(pRunEnv, 0);

    try {
      return Expression.parseStrict(strValue, this.paramList.get(0).getLine(), this.paramList.get(0).getCol(), pRunEnv);
    } catch (ExpressionParseException e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
      // return "";
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);
    checkParameterForString(0,pRunEnv);
    //return new int[] {Types.RET_TYPE_EXPRESSION };
    return new Object[]{ Expression.class };
  }// end method getCheckReturnTypes

} // end class
