package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Expression;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * compileTemplate( &lt;valueString&gt; ) . <br>
 * bersetzungsfunktion-Function fr CnC-Templates.<br>
 * <pre>
 *   &lt;% ...scriptlet code... %&gt;
 *
 *  &lt;%= ...scriptlet expression... %&gt;
 * </pre>
 *
 */
public class CompileTemplateFunction extends AbstractFunction {

  /**
   * constructor
   */
  public CompileTemplateFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
      String strValue = getStringParamNoNull(pRunEnv, 0);

    try {
      return Expression.compileTemplate(strValue);
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      //return "";
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    return checkParameterForString(0, pRunEnv);

    //return new Object[]{ String.class };
  }// end method getCheckReturnTypes

} // end class
