package de.cnc.expression.functions;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * dayOfWeek( &lt;dateValue&gt; ) . <br>
 * Liefert die Tagesinformation innerhalb einer Woche
 * beginnend mit Sonntag (1).
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @see java.util.Calendar
 * @author Heiner Kcker
 * @version $Id: DayOfWeekFunction.java,v 1.3 2005/05/04 12:41:48 hkuecker Exp $
 */
public class DayOfWeekFunction 
    extends AbstractFunction 
{

  /**
   * constructor
   */
  public DayOfWeekFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) 
    throws ExpressionEvaluationException 
  {
    Date inputDate = getDateParamNoNull(pRunEnv, 0);

    try {
        final Integer retInteger = dateToDayOfWeekIntgr(inputDate);
        return retInteger;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
    }

  }// end method eval

  /**
   * @param pDate
   * @return
   */
  private static Integer dateToDayOfWeekIntgr(Date pDate) {
      final GregorianCalendar cal = new GregorianCalendar();
      cal.setTime( pDate );
      int dayOfWeekInt = cal.get( Calendar.DAY_OF_WEEK );
      final Integer retInteger = new Integer( dayOfWeekInt );
      return retInteger;
  }

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    checkParameterForDate(0, pRunEnv);

    //return new Object[]{ Number.class };
    return new Object[]{ Number.class };
  }

} // end class
