package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Expression;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * eval( &lt;valueExpression/expressionString&gt; ) . <br>
 * Fhrt die bergebenene kompilierte Expression aus oder<br>
 * kompiliert und evaluiert den bergebenen String.<br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: EvalFunction.java,v 1.1 2005/04/12 10:09:20 hkuecker Exp $
 */
public class EvalFunction extends AbstractFunction {

  /**
   * constructor
   */
  public EvalFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    // Expression expValue = getExpressionParamNoNull(pRunEnv,0);

    Object objValue = getObjectParamNoNull(pRunEnv, 0);
    Expression expValue = null;

    if (objValue instanceof Expression) {
      // Expression erhalten
      expValue = (Expression) objValue;
    }
    else if (objValue instanceof String) {
      // String erhalten, erst mal kompilieren
      try {
        expValue = Expression.parseStrict((String) objValue, this.paramList.get(0).getLine(), this.paramList.get(0).getCol(), pRunEnv);
      } catch (Exception e) {
        // System.out.println( ""+e ) ;
        throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
        // return "";
      }
    }
    else {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "parameter is no Expression or String");
    }

    try {
      // return Expression.parseStrict( strValue , paramList.get(0).getLine() , paramList.get(0).getCol() ) ;
      return expValue.eval(pRunEnv);
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
      // return "";
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);
    checkParameterForExpressionOrString(0,pRunEnv);
    //return new int[] {Types.RET_TYPE_UNKNOWN };
    //TODO wenn Expression, dann Untersuchung auf Rckgabetyp
    return new Object[]{ Types.UNKNOWN_CLASS };
  }// end method getCheckReturnTypes

} // end class
