package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 *  if( &lt;booleanExpression&gt; , &lt;trueExpression&gt; , &lt;falseExpression&gt; ) . <br>
 * iif( &lt;booleanExpression&gt; , &lt;trueExpression&gt; , &lt;falseExpression&gt; ) . <br>
 * mit verkrzter Auswertung (Short-Circuit-Evaluation)<br>
 */
public class IfFunction extends AbstractFunction {

  /**
   * constructor
   */
  public IfFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    boolean bFirst = getBooleanParamNoNull(pRunEnv, 0).booleanValue();

    if (bFirst) {
      return getObjectParamNullable(pRunEnv, 1);
    }
    else {
      return getObjectParamNullable(pRunEnv, 2);
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(3);

    checkParameterForBoolean(0,pRunEnv);

    //return new int[] {Types.RET_TYPE_UNKNOWN };
//    return new Object[] {
//            this.paramList.get(1).getCheckReturnTypes(pRunEnv) ,
//            this.paramList.get(1).getCheckReturnTypes(pRunEnv) 
//    }
    return new Object[]{ Types.UNKNOWN_CLASS };
  }// end method getCheckReturnTypes

} // end class
