package de.cnc.expression.functions;

import java.lang.reflect.Method;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.util.BeanUtil;
import de.cnc.util.MethodUtil;

/**
 * <b>invokeMethod</b>( &lt;object&gt; , &lt;methodName (String)&gt; [ , &lt;value/values&gt; ] )<br>
 * Aufruf der spezifizierten Methode auf dem bergebenen Objekt mit 
 * den bergebenen Parametern per Reflection <br>
 */
public class InvokeMethodFunction extends AbstractFunction
{

   /**
    * constructor
    */
   public InvokeMethodFunction(String paStrOriginalSource, int paIntLine, int paIntCol)
   {
      super(paStrOriginalSource, paIntLine, paIntCol);
   } // end constructor

   /**
    * process the token
    */
   public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
   {
       if (! pRunEnv.isInvokeEnabled() )
       {
           throw new ExpressionEvaluationException(
                   getLine(),
                   getCol(),
                   getSource(),
                   getOriginalSource(),
                   "method invoke not enabled");
       }
      Object invokeObj = null;

      try
      {
         invokeObj = this.paramList.get(0).eval(pRunEnv);
      }
      catch (Exception e)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "first parameter missed " + e);
      }
      if (invokeObj == null)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "first parameter missed");
      }
      String methodName = getStringParamNoNull(pRunEnv, 1);
      
      //Parameter mit optionaler Anzahl auslesen
      int invokeParamCount = this.paramList.size() - 2;
      Object[] paramValueArr = new Object[ invokeParamCount ];
      for (int i = 0; i < paramValueArr.length; i++)
      {
         paramValueArr[ i ] = this.paramList.get( i  + 2 ).eval(pRunEnv);
      }

      //Erzeugen Class-Array fr Parameter-Klassen
      Class[] paramClassArr = BeanUtil.createParamClassArr(paramValueArr);
      
      //hier funktioniert noch nicht alles sauber, was ist wenn der Parameter null ist
      //bei primitiven Parameter nicht erlaubt, bei nicht primitiven alle erlaubt
      Method method = BeanUtil.getMethod( invokeObj , methodName , paramClassArr );
      
      if (method == null)
      {
         String prmClassStr = BeanUtil.createParamClassStr(paramClassArr);
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "method " + invokeObj.getClass().getName() + "#" + methodName + "( " + prmClassStr + " )" + " not found");
      }
      
      try
      {
         Object retValue = MethodUtil.invoke( method , invokeObj , paramValueArr );

         return retValue;
      }
      catch (Exception e)
      {
         throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
      }

   } // end method eval


  /**
    * get the wahrscheinlichen return typen . <br>
    * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
    * null wenn kein Rckgabe-Wert,
    * mehrere wenn mehrere mglich
    * ( z.B. Plus-Operator kann numerisch oder String liefern)
    */
   public final Object[] getCheckReturnTypes(//--
           final AbstractRuntimeEnvironment pRunEnv)
       throws ExpressionParseException
   {
      checkParamCount(2, 999); // mindestens zwei, aber beliebig viele Parameter
      checkParameterForVariable(0,pRunEnv);
      checkParameterForString(1, pRunEnv);

      //return new int[] { Types.RET_TYPE_UNKNOWN };
      return new Object[]{ Types.UNKNOWN_CLASS };
   } // end method getCheckReturnTypes

} // end class
