package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * isBool( &lt;objectvalue&gt; ) . <br>
 * isBoolean( &lt;objectvalue&gt; ) . <br>
 * isbool( &lt;objectvalue&gt; ) . <br>
 * isboolean( &lt;objectvalue&gt; ) . <br>
 * ermittelt ob der per Parameter bergebene Ausdruck vom Typ Boolean ist
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: IsBoolFunction.java,v 1.1 2005/04/12 10:09:17 hkuecker Exp $
 */
public class IsBoolFunction extends AbstractFunction {

  /**
   * constructor
   */
  public IsBoolFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Object objValue = getObjectParamNullable(pRunEnv,0);

    return (objValue instanceof Boolean)  ? Boolean.TRUE : Boolean.FALSE;
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    //return new int[] {Types.RET_TYPE_BOOL };
    return new Object[]{ Boolean.class };
  }// end method getCheckReturnTypes

} // end class
