package de.cnc.expression.functions;


import java.util.Arrays;
import java.util.Collection;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * emptyStr( &lt;expression&gt; ) . <br>
 * Prfen auf Null, Leer-String, Array mit der Lnge 0 oder leere Collection.
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: IsEmptyFunction.java,v 1.1 2005/04/12 10:09:23 hkuecker Exp $
 */
public class IsEmptyFunction extends AbstractFunction {

  /**
   * constructor
   */
  public IsEmptyFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
//    Object objValue = null;
//
//    try {
//      objValue = paramList.get(0).eval(pRunEnv);
//    } 
//    catch (Exception e) 
//    {
//       // ist es berhaupt sinnvoll hier eine Exception zu werfen ???
//       throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "first parameter missed " + e);
//    }

      Object objValue = getObjectParamNullable(pRunEnv, 0);
      
    try 
    {
       if (objValue == null)
       {
          return Boolean.TRUE;
       }
       if (objValue instanceof String && ( (String) objValue ).length() < 1 )
       {
          return Boolean.TRUE;
       }
       if (objValue instanceof Collection && ( (Collection) objValue ).size() < 1 )
       {
          return Boolean.TRUE;
       }
       if (objValue.getClass().isArray() && Arrays.asList(new Object[0]).size() < 1)
       {
          return Boolean.TRUE;
       }
       return Boolean.FALSE;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      // return Boolean.true;
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    //return new int[] {Types.RET_TYPE_BOOL };
    return new Object[]{ Boolean.class };
  }// end method getCheckReturnTypes

} // end class
