package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * isNullOrSpaceString( &lt;objectvalue&gt; ) . <br>
 * ermittelt ob der per Parameter bergebene Ausdruck null oder ein Leerstring (Lnge 0) ist
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: IsNullOrSpaceStringFunction.java,v 1.1 2005/04/12 10:09:22 hkuecker Exp $
 */
public class IsNullOrSpaceStringFunction extends AbstractFunction {

  /**
   * constructor
   */
  public IsNullOrSpaceStringFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Object objValue = null;

    try {
      objValue = this.paramList.get(0).eval(pRunEnv);
    } catch (Exception e) 
    {
       // ist es berhaupt sinnvoll hier eine Exception zu werfen ???
       throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "first parameter missed " + e);
    }

    try 
    {
       if (objValue == null)
       {
          return Boolean.TRUE;
       }
       if (objValue instanceof String && ( (String) objValue ).trim().length() < 1 )
       {
          return Boolean.TRUE;
       }
       return Boolean.FALSE;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      // return Boolean.true ;
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    //return new int[] {Types.RET_TYPE_BOOL };
    return new Object[]{ Boolean.class };
  }// end method getCheckReturnTypes

} // end class
