package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Expression;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * isValid( &lt;valueString&gt; ) . <br>
 * Prft, ob der bergebene String ohne Fehler kompiliert werden kann.<br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: IsValidFunction.java,v 1.1 2005/04/12 10:09:15 hkuecker Exp $
 */
public class IsValidFunction extends AbstractFunction {

  /**
   * constructor
   */
  public IsValidFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    String strValue = getStringParamNoNull(pRunEnv, 0);

    try {
      Expression.parseStrict(strValue, this.paramList.get(0).getLine(), this.paramList.get(0).getCol(), pRunEnv);
      // kompilieren hat geklappt
      return Boolean.TRUE ;
    } catch (ExpressionParseException e) {// System.out.println( ""+e ) ;
      // throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
      // return "";
    }
    // kompilieren hat nicht geklappt
    return Boolean.FALSE ;

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);
    checkParameterForString(0, pRunEnv);
    //return new int[] {Types.RET_TYPE_EXPRESSION };
    return new Object[]{ Boolean.class };
  }// end method getCheckReturnTypes

} // end class
