package de.cnc.expression.functions;


import java.math.BigDecimal;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * lastIndexOf( &lt;valueString&gt; , &lt;startString&gt; [ , &lt;endIndex(base0)&gt; ] ) . <br>
 * rightAt oder lastPos oder lastIndexOf
 * lastIndexOf-Function like java.lang.String#lastIndexOf()
 */
public class LastIndexOfFunction extends AbstractFunction {

  /**
   * constructor
   */
  public LastIndexOfFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    String strValue = getStringParamNoNull(pRunEnv, 0);

    String strSearch = getStringParamNoNull(pRunEnv, 1);

    if (this.paramList.size() < 3) {

      try {
        return new BigDecimal(strValue.lastIndexOf(strSearch));
      } catch (Exception e) {
        // System.out.println( ""+e ) ;
        throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
        // return new BigDecimal(-1);
      }
    }

    Number numFrom = getNumberParamNoNull(pRunEnv, 2);

    try {
      return new BigDecimal(strValue.lastIndexOf(strSearch, numFrom.intValue()));
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
      // return new BigDecimal(-1);
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    // if (paramList.size() < 2 || paramList.size() > 3 ) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "2 or 3 parameters expected");
    // }
    checkParamCount(2, 3);

    // int[] iTypesArr = paramList.get(0).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "string as first parameter expected");
    // }

    checkParameterForString(0, pRunEnv);

    // iTypesArr = paramList.get(1).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "string as second parameter expected");
    // }
    checkParameterForString(1, pRunEnv);

    if (this.paramList.size() > 2) {

      // iTypesArr = paramList.get(2).getCheckReturnTypes();
      // if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr))) {
      // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "number as 3rd parameter expected");
      // }
      checkParameterForNumber(2, pRunEnv);

    }
    //return new int[] {Types.RET_TYPE_BOOL };
    return new Object[]{ Number.class };
  }// end method getCheckReturnTypes

} // end class
