package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * left( &lt;string value&gt; , &lt;numCount(base1)&gt; ) . <br>
 * liefert numCount Zeichen des Strings von links beginnend
 */
public class LeftFunction extends AbstractFunction {

  /**
   * constructor
   */
  public LeftFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(//--
          final AbstractRuntimeEnvironment pRunEnv)
       throws ExpressionEvaluationException
  {
    final String strValue = getStringParamNoNull(pRunEnv, 0);

    final Number numCount = getNumberParamNoNull(pRunEnv, 1);

    try 
    {
      return strValue.substring(0, Math.min(numCount.intValue() , strValue.length()))  ;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      //return "";
      //TODO implementieren Vorgehensweise vom Clipper wenn from-pos und to-pos nicht ok sind, einfach vorhandenen rest-string zurckgeben
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    // if (paramList.size() < 2 || paramList.size() > 3) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "2 or 3 parameters expected");
    // }
    checkParamCount(2, 3);

    // int[] iTypesArr = paramList.get(0).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "string as first parameter expected");
    // }
    checkParameterForString(0, pRunEnv);

    // iTypesArr = paramList.get(1).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "number as second parameter expected");
    // }
    checkParameterForNumber(1, pRunEnv);

    if (this.paramList.size() > 2) {

      // iTypesArr = paramList.get(1).getCheckReturnTypes();
      // if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr))) {
      // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "number as 3rd parameter expected");
      // }
      checkParameterForNumber(2, pRunEnv);
    }

    //return new Object[]{ String.class };
    return new Object[]{ String.class };
  }// end method getCheckReturnTypes

} // end class
