package de.cnc.expression.functions;


import java.math.BigDecimal;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * max( &lt;valueNumber&gt; , &lt;valueNumber&gt; ) . <br>
 * liefert das Maximum zweier als Parameter bergebenen Werte
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: MaxFunction.java,v 1.1 2005/04/12 10:09:23 hkuecker Exp $
 */
public class MaxFunction extends AbstractFunction {

  /**
   * constructor
   */
  public MaxFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Number numValue1 = getNumberParamNoNull(pRunEnv, 0);
    Number numValue2 = getNumberParamNoNull(pRunEnv, 1);

    try {

      return new BigDecimal(Math.max(numValue1.doubleValue() , numValue2.doubleValue())) ;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      return "";
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(2);

    checkParameterForNumber(0, pRunEnv);
    checkParameterForNumber(1, pRunEnv);

    //return new Object[]{ Number.class };
    return new Object[]{ Number.class };
  }// end method getCheckReturnTypes

} // end class
