package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * ?? noch fertig amchen ?? newStringArray( &lt;numSize&gt; )<br>
 * newStringArray( &lt;str[0]&gt; [ , &lt;str[n]&gt; ] )<br>
 * Erzeugt ein
 *
 *  Objekt
 *
 * neues String-Array in der verlangten Grsse (numerischer Parameter)
 * oder aus der bergebenen String-Parameter-Liste.<br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: NewObjectFunction.java,v 1.1 2005/04/12 10:09:14 hkuecker Exp $
 */
public class NewObjectFunction extends AbstractFunction
{

   /**
    * constructor
    */
   public NewObjectFunction(String paStrOriginalSource, int paIntLine, int paIntCol)
   {
      super(paStrOriginalSource, paIntLine, paIntCol);
   }// end constructor

   /**
    * process the token
    */
   public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
   {
      String strValue = getStringParamNoNull(pRunEnv, 0);

         Class cla = null;

         try
         {
            cla = Class.forName(strValue);
         }
         catch (ClassNotFoundException cnfe)
         {
            throw new ExpressionEvaluationException(getLine(), getCol(), getSource(),
                  getOriginalSource(), cnfe.getMessage() + " " + strValue);
         }
         try
         {
            Object retObject = cla.newInstance();

            // evtl. spter mit mehreren Parametern
            return retObject;
         }
         catch (IllegalAccessException iae)
         {
            throw new ExpressionEvaluationException(getLine(), getCol(), getSource(),
                  getOriginalSource(), iae.getMessage() + " " + cla);
         }
         catch (InstantiationException iae)
         {
            throw new ExpressionEvaluationException(getLine(), getCol(), getSource(),
                  getOriginalSource(), iae.getMessage() + " " + cla);
         }
   }// end method eval

   /**
    * get the wahrscheinlichen return typen . <br>
    * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
    * null wenn kein Rckgabe-Wert,
    * mehrere wenn mehrere mglich
    * ( z.B. Plus-Operator kann numerisch oder String liefern)
    */
   public final Object[] getCheckReturnTypes(//--
           final AbstractRuntimeEnvironment pRunEnv)
       throws ExpressionParseException
   {
      checkParamCount(1);
      checkParameterForString(0, pRunEnv);
      //return new int[] {Types.RET_TYPE_UNKNOWN };
      return new Object[]{ Types.UNKNOWN_CLASS };
   }// end method getCheckReturnTypes

} // end class
