package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * newStringArray( &lt;numSize&gt; )<br>
 * newStringArray( &lt;str[0]&gt; [ , &lt;str[n]&gt; ] )<br>
 * Erzeugt ein neues String-Array in der verlangten Grsse (numerischer Parameter)
 * oder aus der bergebenen String-Parameter-Liste.<br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: NewStringArrayFunction.java,v 1.1 2005/04/12 10:09:23 hkuecker Exp $
 */
public class NewStringArrayFunction extends AbstractFunction {

  /**
   * constructor
   */
  public NewStringArrayFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    // String strValue = getStringParamNoNull(pRunEnv, 0);

    Object objValue = getObjectParamNullable(pRunEnv, 0);

    if (objValue instanceof Number) {
      Number numValue = (Number) objValue;

      return new String[ numValue.intValue() ];
    }
    else if (objValue instanceof String) {
      String[] retStrArr = new String[ this.paramList.size() ];

      for (int i = 0; i < this.paramList.size(); i++) {
        retStrArr[i] = getStringParamNoNull(pRunEnv, i);
      }
      return retStrArr;
    }
    else {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "number or string parameter expected " + objValue);

    }

    // try {
    // return Expression.parseStrict(strValue, paramList.get(0).getLine(), paramList.get(0).getCol());
    // } catch (ExpressionParseException e) {
    // // System.out.println( ""+e ) ;
    // throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
    // // return "";
    // }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCountMinimum(1);
    if (this.paramList.size() == 1) {
      checkParameterForStringOrNumber(0, pRunEnv);
    }
    else {
      for (int i = 0; i < this.paramList.size(); i++) {
        checkParameterForString(i, pRunEnv);
      }
    }
    //return new int[] {Types.RET_TYPE_STRING_ARRAY };
    return new Object[]{ String[].class };
  }// end method getCheckReturnTypes

} // end class
