package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * padl( &lt;string value&gt; , &lt;numCount(base1)&gt; ) . <br>
 * verlngert den als Parameter bergebenen String auf die Lnge numCount
 * durch Auffllen mit Leerzeichen am linken Rand
 */
public class PadlFunction extends AbstractFunction {

  /**
   * constructor
   */
  public PadlFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(//--
          final AbstractRuntimeEnvironment pRunEnv)
    throws ExpressionEvaluationException {
    String strValue = getStringParamNoNull(pRunEnv,0);

    final Number numCount = getNumberParamNoNull(pRunEnv, 1);

    try {
      return spaces(numCount.intValue() - strValue.length()) + strValue;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      return "";
    }

  }// end method eval

  /**
   * liefert eine gewnschte Anzahl Leerzeichen
   */
  private static String spaces(int paIntCount) {
    StringBuffer sb = new StringBuffer();

    for (int i = 0; i < paIntCount; i++) {
      sb.append(' ');
    }
    return sb.toString();
  }// end methoh spaces

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    // if (paramList.size() < 2 || paramList.size() > 3) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "2 or 3 parameters expected");
    // }
    checkParamCount(2, 3);

    // int[] iTypesArr = paramList.get(0).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "string as first parameter expected");
    // }
    checkParameterForString(0, pRunEnv);

    // iTypesArr = paramList.get(1).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "number as second parameter expected");
    // }
    checkParameterForNumber(1, pRunEnv);

    if (this.paramList.size() > 2) {

      // iTypesArr = paramList.get(1).getCheckReturnTypes();
      // if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr))) {
      // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "number as 3rd parameter expected");
      // }
      checkParameterForNumber(2, pRunEnv);
    }

    //return new Object[]{ String.class };
    return new Object[]{ String.class };
  }// end method getCheckReturnTypes

} // end class
