package de.cnc.expression.functions;


import java.math.BigDecimal;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * pow( &lt;valueNum1&gt; , &lt;valueNum2&gt; ) . <br>
 * Potenzieren.<br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: PowerFunction.java,v 1.1 2005/04/12 10:09:19 hkuecker Exp $
 */
public class PowerFunction extends AbstractFunction {

  /**
   * constructor
   */
  public PowerFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Number numValue1 = getNumberParamNoNull(pRunEnv, 0);
    Number numValue2 = getNumberParamNoNull(pRunEnv, 1);

    try {
      double powValue = Math.pow( numValue1.doubleValue() , numValue2.doubleValue() ) ;
      return new BigDecimal(powValue) ;
    } catch (Exception e) {
       //Log.fatal( e );
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      //return new BigDecimal(0);
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(2);

    checkParameterForNumber(0, pRunEnv);
    checkParameterForNumber(1, pRunEnv);

    return new Object[]{ Number.class };
  }// end method getCheckReturnTypes

} // end class
