package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.expression.util.OutPrintStream;


/**
 * println( &lt;objectvalue&gt; ) . <br>
 * Ausgabe des als Parameter bergebenen Ausdruckes auf System.out mit Zeilenumbruch
 * Zurckgegeben wird der ausgegebene Ausdruck.<br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: PrintlnFunction.java,v 1.1 2005/04/12 10:09:17 hkuecker Exp $
 */
public class PrintlnFunction extends AbstractFunction {

  /**
   * constructor
   */
  public PrintlnFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Object objValue = getObjectParamNullable(pRunEnv, 0);

    // System.out.println( "" + objValue ) ;
    // umgelenkter Stream fr Online Tester
    OutPrintStream.getOut().println("" + objValue);
    return objValue;
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    //return new int[] {Types.RET_TYPE_BOOL };
    return new Object[]{ Boolean.class };
  }// end method getCheckReturnTypes

} // end class
