package de.cnc.expression.functions;

import java.util.List;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.util.StringUtil;

/**
 * removeAt( &lt;List or Array&gt; , &lt;value&gt; [ , &lt;index&gt; ] ) . <br>
 * Entfernen eines Wertes aus einer {@link java.util.List} oder einem Array 
 * an der bergebenen Position.
 */
public class RemoveAtFunction extends AbstractFunction
{

   /**
    * constructor
    */
   public RemoveAtFunction(String paStrOriginalSource, int paIntLine, int paIntCol)
   {
      super(paStrOriginalSource, paIntLine, paIntCol);
   } // end constructor

   /**
    * process the token
    */
   public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
   {
//     Object oListOrArray = paramList.get(0).eval(pRunEnv);
//     if (oListOrArray == null)
//     {
//        throw new ExpressionEvaluationException(
//           getLine(),
//           getCol(),
//           getSource(),
//           getOriginalSource(),
//           "first parameter missed");
//     }
     Object oListOrArray = getObjectParamNoNull(pRunEnv, 0);

      Number listIndex = getNumberParamNoNull(pRunEnv, 1);
      //Object delInxObj = paramList.get(1).eval(pRunEnv);

      if ( oListOrArray instanceof List )
      {
         List oList = (List) oListOrArray;
         oList.remove( listIndex.intValue() );
         return oList;
      }
      if ( oListOrArray.getClass().isArray() )
      {
         Object[] oArr = (Object[]) oListOrArray;
         Object[] retArr = StringUtil.arrRemove( oArr , listIndex.intValue() );
         return retArr;
      }
      else
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "list or array expected");
      }

      //try
      //{
      //   Object oValue = paramList.get(1).eval(pRunEnv);

      //   return oValue;
      //}
      //catch (Exception e)
      //{
      //   throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
      //}

   } // end method eval

   /**
    * get the wahrscheinlichen return typen . <br>
    * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
    * null wenn kein Rckgabe-Wert,
    * mehrere wenn mehrere mglich
    * ( z.B. Plus-Operator kann numerisch oder String liefern)
    */
   public final Object[] getCheckReturnTypes(//--
           final AbstractRuntimeEnvironment pRunEnv)
       throws ExpressionParseException
   {
      checkParamCount(2);

      final Object[] typArr = checkParameterForListOrArray(0, pRunEnv);

      checkParameterForNumber(1, pRunEnv);
      
      //return new int[] { Types.RET_TYPE_LIST_OR_ARRAY };
      return typArr;
   } // end method getCheckReturnTypes

} // end class
