package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Assignable;
import de.cnc.expression.Expression;
import de.cnc.expression.SubExpression;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * removeVariable( variable )                                                         <br>
 * removeVar( variable )                                                    <br>
 * Lscht eine als literaler Variablen-Name zur Kompilationszeit bekannte Variable  <br>
 */
public class RemoveVariableFunction extends AbstractFunction {

  /**
   * constructor
   */
  public RemoveVariableFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token . <br>
   * remove variable
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Assignable assignable = null;

    try {
//TODO
      Expression expression = (Expression) this.paramList.get(0);
      SubExpression subExpression = (SubExpression) expression.getFirstSubExpression();

      assignable = (Assignable) subExpression.getToken();

    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "first parameter is no variable " + e);
    }
    if (assignable == null) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "first parameter missed");
    }
    try {
        assignable.remove(pRunEnv);

      return null;
    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);
    checkParameterForVariable(0, pRunEnv);
    //return new int[] {Types.RET_TYPE_NULL };
    return new Object[]{ Types.UNKNOWN_CLASS };
  }// end method getCheckReturnTypes

} // end class
