package de.cnc.expression.functions;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.util.BeanUtil;

/**
 * <b>setProperty</b>( &lt;object&gt; , &lt;propertyName (String)&gt; , &lt;value&gt; )<br>
 * Setzen eines Public-Feldes oder Aufruf der entsprechenden Public-setXy()-Methode per Reflection <br>
 */
public class SetPropertyFunction extends AbstractFunction
{

   /**
    * constructor
    */
   public SetPropertyFunction(String paStrOriginalSource, int paIntLine, int paIntCol)
   {
      super(paStrOriginalSource, paIntLine, paIntCol);
   } // end constructor

   /**
    * process the token
    */
   public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
   {
      Object bean = null;

      try
      {
         bean = this.paramList.get(0).eval(pRunEnv);
      }
      catch (Exception e)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "first parameter missed " + e);
      }
      if (bean == null)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "first parameter missed");
      }
      try
      {
         String propertyName = getStringParamNoNull(pRunEnv, 1);
         Object value = this.paramList.get(2).eval(pRunEnv);
         BeanUtil.set(bean, propertyName, value);
         return value;
      }
      catch (Exception e)
      {
         throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
      }

   } // end method eval

   /**
    * get the wahrscheinlichen return typen . <br>
    * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
    * null wenn kein Rckgabe-Wert,
    * mehrere wenn mehrere mglich
    * ( z.B. Plus-Operator kann numerisch oder String liefern)
    */
   public final Object[] getCheckReturnTypes(//--
           final AbstractRuntimeEnvironment pRunEnv)
       throws ExpressionParseException
   {
      checkParamCount(3);
      checkParameterForVariable(0, pRunEnv);
      checkParameterForString(1, pRunEnv);

      //return new int[] { Types.RET_TYPE_UNKNOWN };
      return new Object[]{ Types.UNKNOWN_CLASS };
   } // end method getCheckReturnTypes

} // end class
