package de.cnc.expression.functions;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.Assignable;
import de.cnc.expression.SubExpression;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;

/**
 * setVarForName( variable , &lt;expression&gt; )                                                         <br>
 * setVariableForName( variable , &lt;expression&gt; )                                                    <br>
 * Setzt eine als literaler Variablen-Name zur Kompilationszeit bekannte Variable  <br>
 */
public class SetVariableForNameFunction extends AbstractFunction
{

    /**
     * constructor
     */
    public SetVariableForNameFunction(
        String paStrOriginalSource,
        int paIntLine,
        int paIntCol)
    {
        super(paStrOriginalSource, paIntLine, paIntCol);
    } // end constructor

    /**
     * process the token . <br>
     * set variable
     */
    public final Object eval(final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionEvaluationException
    {

        String varName = getStringParamNoNull(pRunEnv, 0);

        //System.out.println( "SetVariableForNameFunction#eval varName: " + varName);

        try
        {
            // 2th parameter
            Object oValue = this.paramList.get(1).eval(pRunEnv);

            final AbstractToken setExpression = parseAssignable(varName, getLine(), getCol(), getOriginalSource());
            
            //varName.set(pRunEnv, paObjValue);
            Assignable assignable = (Assignable) setExpression;
            assignable.set(pRunEnv, oValue);

            return oValue;
        }
        catch (Exception e)
        {
            throw new ExpressionEvaluationException(
                getLine(),
                getCol(),
                getSource(),
                getOriginalSource(),
                "" + e);
        }

    } // end method eval

    /**
     * @param paStrVarName
     * @param paIntLine
     * @param paIntCol
     * @param paStrOriginalSource
     * @return
     * @throws ExpressionEvaluationException
     */
    public static AbstractToken parseAssignable(//--
            final String paStrVarName, //--
            final int paIntLine, //--
            final int paIntCol, //--
            final String paStrOriginalSource) 
    throws ExpressionEvaluationException
    {
        final AbstractToken setExpression;

        try
        {
            //Parsen Variable, IndexOperator oder Dot-Operator
            setExpression =
                SubExpression.parseSubExpression(
                    paStrVarName,
                    paIntLine,
                    paIntCol,
                    paStrOriginalSource);
        
            //hier organisieren der Operator-Prioritt, weil der Variable-Name mit eventuellen Indizes erst zur Laufzeit geparst wird
            //varName.organizeOperatorPriority();
            setExpression.organizeOperatorPriority();
        }
        catch (ExpressionParseException expParExc)
        {
            throw new ExpressionEvaluationException(
                paIntLine,
                paIntCol,
                paStrVarName,
                paStrOriginalSource,
                expParExc.toString());
        }
        //if (varName == null)

//System.out.println( "setExpression: " + setExpression.getClass().getName() );
        
        if (setExpression == null 
                || ! (setExpression instanceof Assignable) )
        {
            throw new ExpressionEvaluationException(
                paIntLine,
                paIntCol,
                paStrVarName,
                paStrOriginalSource,
                //"string " + paStrVarName + " is no variable name");
                "string " + paStrVarName + " is no variable or indexed expression");
        }
        return setExpression;
    }

    /**
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    public final Object[] getCheckReturnTypes(//--
            final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionParseException
    {
        // if (paramList.size() != 2) {
        // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "2 parameters expected");
        // }

        checkParamCount(2);

        // int[] iTypesArr = paramList.get(0).getCheckReturnTypes();
        // if (!(Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr))) {
        // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "string as first parameter expected");
        // }

        checkParameterForString(0, pRunEnv);

        //return new int[] { Types.RET_TYPE_UNKNOWN };
        return new Object[]{ Types.UNKNOWN_CLASS };
    } // end method getCheckReturnTypes

} // end class
