package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Assignable;
import de.cnc.expression.Expression;
import de.cnc.expression.SubExpression;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.util.Log;


/**
 * setVar( variable , &lt;expression&gt; )                                                         <br>
 * setVariable( variable , &lt;expression&gt; )                                                    <br>
 * Setzt eine als literaler Variablen-Name zur Kompilationszeit bekannte Variable  <br>
 */
public class SetVariableFunction extends AbstractFunction {

  /**
   * constructor
   */
  public SetVariableFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token . <br>
   * set variable
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Assignable variable = null;

    try {

      Expression expression = (Expression) this.paramList.get(0);
      SubExpression subExpression = (SubExpression) expression.getFirstSubExpression();

      variable = (Assignable) subExpression.getToken();

    } catch (Exception e) {
//ExceptionUtil.printToErr( e );
        Log.fatal( e );
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "1th parameter is no variable " + e);
    }
    if (variable == null) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "first parameter missed");
    }

    // 2th parameter
    try {
      Object oValue = this.paramList.get(1).eval(pRunEnv);

      variable.set(pRunEnv, oValue);

      // Rckgabe des gesetzten Wertes
      return oValue;
    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(2);
    checkParameterForVariable(0, pRunEnv);

    //return new int[] {Types.RET_TYPE_UNKNOWN };
    return new Object[]{ Types.UNKNOWN_CLASS };
  }// end method getCheckReturnTypes

} // end class
