package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * startsWith( &lt;valueString&gt; , &lt;startString&gt; [ , &lt;endIndex(base0)&gt; ] ) . <br>
 * startsWith-Function like java.lang.String.startsWith()
 */
public class StartsWithFunction extends AbstractFunction {

  /**
   * constructor
   */
  public StartsWithFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    String strValue = getStringParamNoNull(pRunEnv, 0);

    String strStart = getStringParamNoNull(pRunEnv, 1);

    if (this.paramList.size() < 3) {

      try {
        return (strValue.startsWith(strStart)) ? Boolean.TRUE : Boolean.FALSE;
      } catch (Exception e) {
        // System.out.println( ""+e ) ;
        // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
        return Boolean.FALSE;
      }

    }

    Number numFrom = getNumberParamNoNull(pRunEnv, 2);

    try {
      return (strValue.startsWith(strStart, numFrom.intValue())) ? Boolean.TRUE : Boolean.FALSE;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      return Boolean.FALSE;
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(2, 3);
    checkParameterForString(0, pRunEnv);
    checkParameterForString(1, pRunEnv);
    if (this.paramList.size() > 2) {
      checkParameterForNumber(2, pRunEnv);
    }
    //return new int[] {Types.RET_TYPE_BOOL };
    return new Object[]{ Boolean.class };
  }// end method getCheckReturnTypes

} // end class
