package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * str( &lt;valueNumber&gt; ) . <br>
 * wandelt einen numerischen Wert in einen String um
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: StrFunction.java,v 1.1 2005/04/12 10:09:21 hkuecker Exp $
 */
public class StrFunction extends AbstractFunction {

  /**
   * constructor
   */
  public StrFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Number numCount = getNumberParamNoNull(pRunEnv, 0);

    try {

      return numCount.toString()  ;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      return "";
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    checkParameterForNumber(0, pRunEnv);

    //return new Object[]{ String.class };
    return new Object[]{ String.class };
  }// end method getCheckReturnTypes

} // end class
