package de.cnc.expression.functions;

import java.util.Date;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.util.Log;
import de.cnc.util.StringUtil;


/**
 * str( &lt;valueNumber&gt; ) . <br>
 * wandelt einen String 'dd.MM.yyyy' in einen String um
 * oder liefert 'null' wenn ungltig.
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: StrToDateFunction.java,v 1.1 2005/04/12 10:09:15 hkuecker Exp $
 */
public class StrToDateFunction 
    extends AbstractFunction 
{

  /**
   * constructor
   */
  public StrToDateFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) 
    throws ExpressionEvaluationException 
  {
    String inputStr = getStringParamNoNull(pRunEnv, 0);

    try {

      return StringUtil.parseDateCatched( inputStr );
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
        Log.fatal( e );
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      //return "";
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    checkParameterForString(0, pRunEnv);

    //return new int[] {Types.RET_TYPE_DATE, Types.RET_TYPE_NULL };
    return new Object[]{ Date.class };
  }// end method getCheckReturnTypes

} // end class
