package de.cnc.expression.functions;

import java.util.Date;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * stripTimeOfDay( &lt;dateValue&gt; ) . <br>
 * Liefert die Datumsinformation innerhalb eines Tages
 * durch Entfernen der Uhrzeit-Information.
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: StripTimeOfDayFunction.java,v 1.1 2005/04/12 10:09:18 hkuecker Exp $
 */
public class StripTimeOfDayFunction 
    extends AbstractFunction 
{

  /**
   * constructor
   */
  public StripTimeOfDayFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) 
    throws ExpressionEvaluationException 
  {
    Date inputDate = getDateParamNoNull(pRunEnv, 0);

    try {
        java.sql.Date dateWithoutTime = new java.sql.Date( inputDate.getTime() );
        return dateWithoutTime;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    checkParameterForDate(0, pRunEnv);

    return new Object[]{ Date.class };
  }

} // end class
