package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * substring( &lt;string value&gt; , &lt;beginIndex(base0)&gt; [ , &lt;endIndex(base0)&gt; ] ) . <br>
 * substring-Function like java.lang.String.substring()
 */
public class SubstringFunction extends AbstractFunction {

  /**
   * constructor
   */
  public SubstringFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionEvaluationException 
  {
      final String strValue = getStringParamNoNull(pRunEnv, 0);

      final Number numFrom = getNumberParamNoNull(pRunEnv, 1);

    // System.out.println( "numFrom " + numFrom ) ;

    if (this.paramList.size() < 3) {
      // string, beginIndex
      try {
        return strValue.substring(numFrom.intValue());
      } catch (Exception e) {
        // System.out.println( ""+e ) ;
        // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
        return "";
      }

    }

    // string, beginIndex, endIndex
    final Number numTo = getNumberParamNoNull(pRunEnv, 2);

    // System.out.println( "numTo " + numTo ) ;
    try {
      return strValue.substring(numFrom.intValue(), numTo.intValue());
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      //return "";
      //TODO implementieren Vorgehensweise vom Clipper wenn from-pos und to-pos nicht ok sind, einfach vorhandenen rest-string zurckgeben
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    // if (paramList.size() < 2 || paramList.size() > 3) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "2 or 3 parameters expected");
    // }
    checkParamCount(2, 3);

    // int[] iTypesArr = paramList.get(0).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "string as first parameter expected");
    // }
    checkParameterForString(0, pRunEnv);

    // iTypesArr = paramList.get(1).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "number as second parameter expected");
    // }
    checkParameterForNumber(1, pRunEnv);

    if (this.paramList.size() > 2) {

      // iTypesArr = paramList.get(1).getCheckReturnTypes();
      // if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr))) {
      // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "number as 3rd parameter expected");
      // }
      checkParameterForNumber(2, pRunEnv);
    }

    return new Object[]{ String.class };
  }// end method getCheckReturnTypes

} // end class
