package de.cnc.expression.functions;

import java.sql.Time;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Date;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * timeOfDay( &lt;dateValue&gt; ) . <br>
 * Liefert die Uhrzeit innerhalb eines Tages
 * durch Entfernen der Datumsinformation.
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @see java.util.Calendar
 * @author Heiner Kcker
 */
public class TimeOfDayFunction 
    extends AbstractFunction 
{

  /**
   * constructor
   */
  public TimeOfDayFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) 
    throws ExpressionEvaluationException 
  {
    Date inputDate = getDateParamNoNull(pRunEnv, 0);

    try {
//        GregorianCalendar calendar = new GregorianCalendar(  );
//        calendar.setTime( inputDate );
//        int hours = calendar.get( Calendar.HOUR_OF_DAY );
//        int minutes = calendar.get( Calendar.MINUTE );
//        int seconds = calendar.get( Calendar.SECOND );
//        int millis = calendar.get( Calendar.MILLISECOND );
        
        //Time time = new Time( inputDate.getTime() % ( 24 * 60 * 60 * 1000 ) );
        
        // Umwandlung Time -> String -> Time, weil sonst nicht korrekt Comparable (Tagesdatum existiert noch im long)
        Time time = dateToTime(inputDate);
        return time;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
    }

  }// end method eval

  /**
   * Umwandlung Time -> String -> Time, weil sonst nicht korrekt
   * Comparable (Tagesdatum existiert noch im long).
   * 
   * @param pDate
   * @return
   */
  public static Time dateToTime(final Date pDate) {
      //return Time.valueOf( new Time( pDate.getTime() ).toString() );
      //SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm:ss.SSS");
      final SimpleDateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");
      final String timeStr = dateFormat.format(pDate);
//System.out.println("timeStr: "+timeStr);
      Time time = new Time( Timestamp.valueOf( "1970-01-01 " + timeStr ).getTime() );
      return time;
  }

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    checkParameterForDate(0,pRunEnv);

    return new Object[]{ Date.class };
  }

} // end class
