package de.cnc.expression.functions;


import java.math.BigDecimal;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * toDegrees( &lt;valueNum&gt; ) . <br>
 * ToDegreesFunction, Umwandlung des als Parameter bergebenen Wertes von Radiant zu Dezimalgrad.<br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: ToDegreesFunction.java,v 1.1 2005/04/12 10:09:22 hkuecker Exp $
 */
public class ToDegreesFunction extends AbstractFunction {

  /**
   * constructor
   */
  public ToDegreesFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    Number numValue = getNumberParamNoNull(pRunEnv, 0);

    try {
      double toDegreesValue = Math.toDegrees( numValue.doubleValue() ) ;
      return new BigDecimal(toDegreesValue) ;
    } catch (Exception e) {
       //Log.fatal( e );
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      //return new BigDecimal(0);
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);

    checkParameterForNumber(0,pRunEnv);

    return new Object[]{ Number.class };
  }// end method getCheckReturnTypes

} // end class
