package de.cnc.expression.functions;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Expression;
import de.cnc.expression.SubExpression;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.expression.tokencomplex.Variable;


/**
 * varExists( variable  ) <br>
 * Ob eine als literaler Variablen-Name zur Kompilationszeit bekannte Variable existiert. <br>
 */
public class VariableExistsFunction extends AbstractFunction {

  /**
   * constructor
   */
  public VariableExistsFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token . <br>
   * set variable
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    boolean bExists = true;

    Variable variable = null;
    Object oVarValue = null;

    try {

      Expression expression = (Expression) this.paramList.get(0);
      SubExpression subExpression = (SubExpression) expression.getFirstSubExpression();

      variable = (Variable) subExpression.getToken();
      // System.out.println("1variable:" + variable);
      oVarValue = variable.eval(pRunEnv);
      // System.out.println("2oVarValue:" + oVarValue);

    } catch (Exception e) {
      System.err.println(e.getMessage()) ;
      // throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "1th parameter is no variable " + e);
      bExists = false;
    }
    if (oVarValue == null) {
      // throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "first parameter missed");
      bExists = false;
    }

    // System.out.println("3bExists:" + bExists);
    return (bExists) ? Boolean.TRUE : Boolean.FALSE;
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    checkParamCount(1);
    checkParameterForVariable(0,pRunEnv);

    return new Object[]{ Boolean.class };
  }// end method getCheckReturnTypes

} // end class
