package de.cnc.expression.infixoperators;


import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * Ampersand Operator . <br>
 * Und-Verknpfung ohne verkrzte Auswertung (Short-Circuit-Evaluation)
 */
public class AndOperator extends AbstractInfixOperator {

  public static AndOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "AndOperator#parse:"+paStr);
    if ("&".equals(paStr)) {
      AndOperator andAndOperator = new AndOperator(paStrOriginalSource, paIntLine, paIntCol);

      andAndOperator.strSource = "&";
      andAndOperator.iSrcLength = 1;
      return andAndOperator;
    }
    // Schutz vor Erkennung &&
    else if (paStr.length() > 1 && paStr.startsWith("&") && paStr.charAt(1) != '&') {
      AndOperator andAndOperator = new AndOperator(paStrOriginalSource, paIntLine, paIntCol);

      andAndOperator.strSource = "&";
      andAndOperator.iSrcLength = 1;
      return andAndOperator;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no && operator");

  }// end method parse

  /**
   * constructor
   */
  public AndOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
    // System.out.println("constructor AndOperator");
    // (new Exception("")).printStackTrace();
  }// end constructor

  /**
   * process the operator
   */
  public final Object eval(final Object paObjLeft, Object paObjRight) throws ExpressionEvaluationException {

    if (paObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(paObjLeft instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; Boolean expected");
    }
    Boolean bLeftValue = (Boolean) paObjLeft;

    if (paObjRight == null) {
      throw new ExpressionEvaluationException(this, "right operand is null");
    }
    if (!(paObjRight instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; Boolean expected");
    }
    Boolean bRightValue = (Boolean) paObjRight;

    // System.out.println("bLeftValue " + bLeftValue);
    // System.out.println("bRightValue " + bRightValue);
    try {
      return (bLeftValue.booleanValue() && bRightValue.booleanValue()) ? Boolean.TRUE : Boolean.FALSE;
     } catch (Exception e) {
      throw new ExpressionEvaluationException(this, "" + e);
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(final Object[] paIntArrLeft, final Object[] paIntArrRight)  throws ExpressionParseException {
    return checkBoolBoolBool(this, paIntArrLeft, paIntArrRight);
  }// end method getCheckReturnTypes

  public int getPriorityLevel() {
    return 1;
  }// end method getPriorityLevel

}// end class
