package de.cnc.expression.infixoperators;


import java.math.BigDecimal;

import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


public class DivideOperator extends AbstractInfixOperator {

  public static DivideOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "DivideOperator#parse:"+paStr);
    DivideOperator divideOp = new DivideOperator(paStrOriginalSource, paIntLine, paIntCol);

    divideOp.strSource = "/";
    divideOp.iSrcLength = 1;

    if ("/".equals(paStr)) {
      return divideOp;
    }
    // Schutz gegen flschliches Erkennen // fr Inline-Kommentar
    else if (paStr.length() > 1 && paStr.startsWith("/") && paStr.charAt(1) != '/') {
      return divideOp;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no divide operator");

  }// end method parse

  /**
   * constructor
   */
  public DivideOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator
   */
  public final Object eval(final Object paObjLeft, Object paObjRight) throws ExpressionEvaluationException {

    if (paObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(paObjLeft instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; BigDecimal expected");
    }

    if (paObjRight == null) {
      throw new ExpressionEvaluationException(this, "right operand is null");
    }
    if (!(paObjRight instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; BigDecimal expected");
    }
    String strLeftValue = "" + paObjLeft;
    String strRightValue = "" + paObjRight;

    // Abfangung division durch Null
    try {
      return (new BigDecimal(strLeftValue)).setScale(6, BigDecimal.ROUND_HALF_EVEN).divide(new BigDecimal(strRightValue),
          BigDecimal.ROUND_HALF_EVEN); // ROUND_HALF_EVEN);
    } catch (Exception e) {
      throw new ExpressionEvaluationException(this, "" + e);
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(final Object[] paIntArrLeft, final Object[] paIntArrRight)  throws ExpressionParseException {
    return checkNumNumNum(this, paIntArrLeft, paIntArrRight);
  }// end method getCheckReturnTypes

  public int getPriorityLevel() {
    return 4;
  }// end method getPriorityLevel
}// end class
