package de.cnc.expression.infixoperators;


import java.math.BigDecimal;

import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.expression.util.DateTimeCompareUtil;
import de.cnc.util.ExceptionUtil;


public class GreaterOrEqualOperator extends AbstractInfixOperator {

  public static GreaterOrEqualOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "GreaterOrEqualOperator#parse:"+paStr);

    if (paStr.startsWith(">=")) {
      GreaterOrEqualOperator greaterOrEqOp = new GreaterOrEqualOperator(paStrOriginalSource, paIntLine, paIntCol);

      greaterOrEqOp.iSrcLength = 2;
      greaterOrEqOp.strSource = ">=";
      return greaterOrEqOp;
    }
    else if (paStr.startsWith("=>")) {
      GreaterOrEqualOperator greaterOrEqOp = new GreaterOrEqualOperator(paStrOriginalSource, paIntLine, paIntCol);

      greaterOrEqOp.iSrcLength = 2;
      greaterOrEqOp.strSource = "=>";
      return greaterOrEqOp;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no greater or equal operator");

  }// end method parse

  /**
   * constructor
   */
  public GreaterOrEqualOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator
   */
  public final Object eval(//--
          Object paObjLeft, //--
          Object paObjRight)
  throws ExpressionEvaluationException 
  {

    if (paObjLeft == null) {
//      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    else if (paObjLeft instanceof Boolean) {
      // dirty trick, Umwandlung in String
      // da t im Alphabet nach f folgt, ist true > false ( 0 > 1 )
      // klappt nur in englisch
      paObjLeft = "" + paObjLeft;
    }
    else if (paObjLeft instanceof Number) {
      // Normieren numerische Werte auf BigDecimal
      paObjLeft = new BigDecimal("" + paObjLeft);
    }
    else if (!(paObjLeft instanceof Comparable)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; Comparable expected");
    }
    Comparable cmpLeftValue = (Comparable) paObjLeft;

    if (paObjRight == null) {
//      throw new ExpressionEvaluationException(this, "right operand is null");
    }
    else if (paObjRight instanceof Boolean) {
      // dirty trick, Umwandlung in String
      // da t im Alphabet nach f folgt, ist true > false ( 0 > 1 )
      // klappt nur in englisch
      paObjRight = "" + paObjRight;
    }
    else if (paObjRight instanceof Number) {
      // Normieren numerische Werte auf BigDecimal
      paObjRight = new BigDecimal("" + paObjRight);
    }
    else if (!(paObjRight instanceof Comparable)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; Comparable expected");
    }
    Comparable cmpRightValue = (Comparable) paObjRight;

    try {
      return ( DateTimeCompareUtil.greaterEqual( cmpLeftValue , cmpRightValue ) ) ? Boolean.TRUE : Boolean.FALSE;
    } catch (Exception e) {
        //ExceptionUtil.printToErr( e );
      throw new ExpressionEvaluationException(this, //-- "" + e);
              ExceptionUtil.getMessageAndStackTrace( e ) );
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final Object[] paIntArrLeft, //--
          final Object[] paIntArrRight)
      throws ExpressionParseException 
  {
    //return new int[] {Types.RET_TYPE_BOOL };
      return new Object[] { Boolean.class };
  }// end method getReturnTypes

  public int getPriorityLevel() {
    return 2;
  }// end method getPriorityLevel
}// end class
