package de.cnc.expression.infixoperators;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * Negiertes UND Operator . <br>
 * Negierte Und-Verknpfung mit verkrzter Auswertung (Short-Circuit-Evaluation)
 */
public class NandOperator extends AbstractInfixOperator {

  public static NandOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "NandOperator#parse:"+paStr);
    if ("nand".equals(paStr) || // ...
        // verhindern verwechseln mit variablenname oder funktionsname
        (paStr.length() > 4 && paStr.startsWith("nand") && !Character.isLetterOrDigit(paStr.charAt(4)))) {
      NandOperator nandOperator = new NandOperator(paStrOriginalSource, paIntLine, paIntCol);

      nandOperator.strSource = "nand";
      nandOperator.iSrcLength = 4;
      return nandOperator;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no && operator");

  }// end method parse

  /**
   * constructor
   */
  public NandOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
    // System.out.println("constructor NandOperator");
    // (new Exception("")).printStackTrace();
  }// end constructor

  /**
   * process the operator with short circuit evaluation . <br>
   *
   * Diese Methode wurde fr die verkrzte Auswertung
   * (short circuit evaluation) berschrieben. Aufpassen, etwas dirty!
   */
  public final Object eval(//--
          Object paObjLeft, AbstractToken paTokRight, AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    if (paObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(paObjLeft instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; Boolean expected");
    }
    Boolean bLeftValue = (Boolean) paObjLeft;

    // verkrzte Auswertung false
    if (!bLeftValue.booleanValue()) {
      return (!bLeftValue.booleanValue()) ? Boolean.TRUE : Boolean.FALSE;
    }

    return eval(paObjLeft, paTokRight.eval(pRunEnv));
  }

  /**
   * process the operator
   */
  public final Object eval(final Object paObjLeft, Object paObjRight) throws ExpressionEvaluationException {

    if (paObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(paObjLeft instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; Boolean expected");
    }
    Boolean bLeftValue = (Boolean) paObjLeft;

    if (paObjRight == null) {
      throw new ExpressionEvaluationException(this, "right operand is null");
    }
    if (!(paObjRight instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; Boolean expected");
    }
    Boolean bRightValue = (Boolean) paObjRight;

    // System.out.println("bLeftValue " + bLeftValue);
    // System.out.println("bRightValue " + bRightValue);
    try {
      return (!(bLeftValue.booleanValue() && bRightValue.booleanValue())) ? Boolean.TRUE : Boolean.FALSE;
    } catch (Exception e) {
      throw new ExpressionEvaluationException(this, "" + e);
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(final Object[] paIntArrLeft, final Object[] paIntArrRight)  throws ExpressionParseException {
    return checkBoolBoolBool(this, paIntArrLeft, paIntArrRight);

  }// end method getCheckReturnTypes

  public int getPriorityLevel() {
    return 1;
  }// end method getPriorityLevel

}// end class
