package de.cnc.expression.infixoperators;


import java.math.BigDecimal;

import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


public class PlusOperator extends AbstractInfixOperator {

  public static PlusOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "PlusOperator#parse:"+paStr);
    PlusOperator plusOp = new PlusOperator(paStrOriginalSource, paIntLine, paIntCol);

    plusOp.strSource = "+";
    plusOp.iSrcLength = 1;

    if ("+".equals(paStr)) {
      return plusOp;
    }
    else if (paStr.length() > 1 && paStr.startsWith("+") && paStr.charAt(1) != '+') {
      return plusOp;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no plus operator");

  }// end method parse

  /**
   * constructor
   */
  public PlusOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator
   */
  public final Object eval(final Object paObjLeft, Object paObjRight) throws ExpressionEvaluationException {

    if (paObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (paObjLeft instanceof Number) {
      String strLeftValue = "" + paObjLeft;

      if (paObjRight == null) {
        throw new ExpressionEvaluationException(this, "right operand is null");
      }
      if (!(paObjRight instanceof Number)) {
        throw new ExpressionEvaluationException(this,
            "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; BigDecimal expected");
      }
      String strRightValue = "" + paObjRight;

      // BigDecimal retBigDec = (new BigDecimal(strLeftValue)).add(new BigDecimal(strRightValue));
      // System.out.println( "add "+strLeftValue+" and "+strRightValue+" to:"+ retBigDec ) ;
      // return retBigDec ;

      return (new BigDecimal(strLeftValue)).add(new BigDecimal(strRightValue));
    }
    else if (paObjLeft instanceof String) {
      // Strings
      String strLeftValue = "" + paObjLeft;

      String strRightValue = "" + paObjRight;

      return strLeftValue + strRightValue;

    }
    else {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; BigDecimal or String expected");
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen .<br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert, mehrere wenn mehrere mglich ( z.B.
   * Plus-Operator kann numerisch oder String liefern)
   */
    public final Object[] getCheckReturnTypes(//--
            final Object[] pTypArrLeft, //--
            final Object[] pTypArrRight)
        throws ExpressionParseException
    {
//Log.debug("pTypArrLeft",pTypArrLeft);
//Log.debug("pTypArrRight",pTypArrRight);

        if (Types.contentsUnknown(pTypArrLeft))
        {
            // man weiss zum kompilierzeitpunkt nicht was fr ein typ kommt
            // es wird zurckgegeben, dass eine String oder eine Zahl kommen
            // knnte
//System.out.println("return 1");
            return new Object[]
            {Number.class, String.class};
        }

        // links wird ein String oder eine zahl erwartet
        // kommt links ein string, ist als rechter operand alles erlaubt, wird
        // java-typisch zum string umgewandelt
        // kommt links eine zahl, wird rechts auch eine zahl verlangt
        if (Types.isInOrUnknown(String.class, pTypArrLeft))
        {
            // links ein String, dann ist rechts jeder Type erlaubt
            // (Java-Vorgehensweise)
//System.out.println("return 2");
            return new Object[]
            {String.class};
        }

        if (Types.isInOrUnknown(Number.class, pTypArrLeft))
        {
            if (Types.isInOrUnknown(Number.class, pTypArrRight))
            {
//System.out.println("return 3");
                return new Object[]
                {Number.class};
            }
            else
            {
                throw new ExpressionParseException(getLine(), getCol(),
                        getOriginalSource(),
                        "numeric type for right value expected");
            }
        }
        throw new ExpressionParseException(getLine(), getCol(),
                getOriginalSource(), "numeric or string type expected");

    }// end method getCheckReturnTypes

    public int getPriorityLevel()
    {
        return 3;
    }// end method getPriorityLevel
}// end class
