package de.cnc.expression.infixoperators;


import java.math.BigDecimal;

import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


public class TimesOperator extends AbstractInfixOperator {

  public static TimesOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "TimesOperator#parse:"+paStr);
    if ("*".equals(paStr) || // ...
        paStr.length() > 1 && paStr.startsWith("*") && paStr.charAt(1) != '*') {
      TimesOperator timesOp = new TimesOperator(paStrOriginalSource, paIntLine, paIntCol);

      timesOp.strSource = "*";
      timesOp.iSrcLength = 1;

      return timesOp;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no times operator");

  }// end method parse

  /**
   * constructor
   */
  public TimesOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator
   */
  public final Object eval(final Object paObjLeft, Object paObjRight) throws ExpressionEvaluationException {

    if (paObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(paObjLeft instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; BigDecimal expected");
    }

    if (paObjRight == null) {
      throw new ExpressionEvaluationException(this, "right operand is null");
    }
    if (!(paObjRight instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; BigDecimal expected");
    }
    String strLeftValue = "" + paObjLeft;
    String strRightValue = "" + paObjRight;

    return (new BigDecimal(strLeftValue)).multiply(new BigDecimal(strRightValue));
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(final Object[] paIntArrLeft, final Object[] paIntArrRight)  throws ExpressionParseException {
    return checkNumNumNum(this, paIntArrLeft, paIntArrRight);
  }// end method getCheckReturnTypes

  public int getPriorityLevel() {
    return 4;
  }// end method getPriorityLevel
}// end class
