package de.cnc.expression.infixoperators;


import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * xor Exclusiv Oder Operator . <br>
 * Exclusiv-Oder-Verknpfung ohne verkrzter Auswertung (Short-Circuit-Evaluation)
 */
public class XorOperator extends AbstractInfixOperator {

  public static XorOperator parse(String paStr, int paIntLine, int paIntCol, String paStrXoriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "XorOperator#parse:"+paStr);
    if ("xor".equals(paStr) || // ...
        // Schutz vor Erkennung variablen oder funktionsname
        (paStr.length() > 3 && paStr.startsWith("xor") && !Character.isLetterOrDigit(paStr.charAt(3)))) {
      XorOperator xorOperator = new XorOperator(paStrXoriginalSource, paIntLine, paIntCol);

      xorOperator.strSource = "xor";
      xorOperator.iSrcLength = 3;
      return xorOperator;
    }
    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no || operator");

  }// end method parse

  /**
   * constructor
   */
  public XorOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
    // System.out.println("constructor XxorOperator");
    // (new Exception("")).printStackTrace();
  }// end constructor

  /**
   * process the operator
   */
  public final Object eval(//--
          final Object paObjLeft, //--
          final Object paObjRight)
  throws ExpressionEvaluationException 
  {

    if (paObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(paObjLeft instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; Boolean expected");
    }
    Boolean bLeftValue = (Boolean) paObjLeft;

    if (paObjRight == null) {
      throw new ExpressionEvaluationException(this, "right operand is null");
    }
    if (!(paObjRight instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; Boolean expected");
    }
    Boolean bRightValue = (Boolean) paObjRight;

    // System.out.println("bLeftValue " + bLeftValue);
    // System.out.println("bRightValue " + bRightValue);
    try {
      return (bLeftValue.booleanValue() != bRightValue.booleanValue()) ? Boolean.TRUE : Boolean.FALSE;
    } catch (Exception e) {
      throw new ExpressionEvaluationException(this, "" + e);
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(final Object[] paIntArrLeft, final Object[] paIntArrRight)  throws ExpressionParseException {
    return checkBoolBoolBool(this, paIntArrLeft, paIntArrRight);
  }// end method getCheckReturnTypes

  public int getPriorityLevel() {
    return 1;
  }// end method getPriorityLevel

}// end class
