package de.cnc.expression.prepostfixoperators;


import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


/**
 * Abstrakte Oberklasse aller Prefix- und Postfix-Operatoren.
 * 
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: AbstractPrePostfixOperator.java,v 1.1 2005/04/12 10:09:06 hkuecker Exp $
 */
public abstract class AbstractPrePostfixOperator {

  protected int iLine;
  protected int iCol;
  protected String strSource;
  protected int iSrcLength = 0;
  protected String strOriginalSource;

  public AbstractPrePostfixOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    this.strOriginalSource = paStrOriginalSource;
    this.iLine = paIntLine;
    this.iCol = paIntCol;
  }

  /**
   * gets the the source line number
   */
  public int getLine() {
    return this.iLine;
  }

  /**
   * gets the col position in the source code line
   */
  public int getCol() {
    return this.iCol;
  }

  public String getSource() {
    return this.strSource;
  }

  public int getSrcLength() {
    return this.iSrcLength;
  }

  public String getOriginalSource() {
    return this.strOriginalSource;
  }

  /**
   * Prfroutine fr Variablen-Namen . <br>
   * @see de.cnc.expression.tokencomplex.Variable#parse
   */
  public static boolean isOperator(String strPa) {
    return
        "!".equals(strPa) //--
        || "-".equals(strPa) //--
        || "++".equals(strPa) ;
  }// end method isOperator

  /**
   * Parse prefix operators like !b, -i, --i and ++i
   *
   * @return geparsten Operator oder null wenn nichts passendes gefunden
   */
  public static AbstractPrePostfixOperator parsePrefixOperator(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException {
    // System.out.println( "parsePrefixOperator:"+paStr);
    // dirty trick
    // Ausnutzung verkrzte Auflsung logischer Ausdrcke
    AbstractPrePostfixOperator retOperator = null;
    boolean bFake = // ...
        ((retOperator = PlusPlusOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
        ((retOperator = MinusMinusOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
        ((retOperator = MinusPrefixOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
        ((retOperator = NotPrefixOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null); // ...

    return retOperator;

  }// end method parsePrefixOperator

  /**
   * Parse postfix operators like i++ and i--
   *
   * @return geparsten Operator oder null wenn nichts passendes gefunden
   */
  public static AbstractPrePostfixOperator parsePostfixOperator(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException {
    // System.out.println( "parsePrefixOperator:"+paStr);
    // dirty trick
    // Ausnutzung verkrzte Auflsung logischer Ausdrcke
    AbstractPrePostfixOperator retOperator = null;
    boolean bFake = // ...
        ((retOperator = PlusPlusOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
        ((retOperator = MinusMinusOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null); // ...

    return retOperator;

  }// end method AbstractPostfixOperator

  /**
   * debug output
   */
  public String toString() {
    String strRet = getClass().getName() + " source: " + getSource();

    return strRet;
  }// end method toString

  /**
   * process the operator
   */
  // public abstract Object eval(Object paValue) throws ExpressionEvaluationException;

  /**
   * process the operator
   */
  public abstract Object eval(AbstractRuntimeEnvironment pRunEnv, AbstractToken   paTok) throws ExpressionEvaluationException;

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  // abstract public final Object[] getCheckReturnTypes();

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  // abstract public final Object[] getCheckReturnTypes()  throws ExpressionParseException ;
//  abstract public Object[] getCheckReturnTypes(//--
//          Object[] pClassArrTypes)
//      throws ExpressionParseException;
  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public Object[] getCheckReturnTypes(final Object[] pArrTypes)
    throws ExpressionParseException {
    if (!(Types.isInOrUnknown(Number.class, pArrTypes))) {
      throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "numeric type expected");
    }
    return new Object[]{ Number.class };
  }// end  method getCheckReturnTypes


}// end class
