package de.cnc.expression.prepostfixoperators;


import java.math.BigDecimal;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.Assignable;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;


public class MinusMinusOperator extends AbstractPrePostfixOperator {

  public static MinusMinusOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "MinusMinusOperator#parse:"+paStr);
    if (paStr.startsWith("--")) {

      MinusMinusOperator minusMinusOp = new MinusMinusOperator(paStrOriginalSource, paIntLine, paIntCol);

      minusMinusOp.strSource = "--";
      minusMinusOp.iSrcLength = 2;

      return minusMinusOp;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no minus operator");

  }// end method parse

  /**
   * constructor
   */
  public MinusMinusOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv, AbstractToken   paTok) throws ExpressionEvaluationException {
    if (!(paTok instanceof Assignable)) {
      throw new ExpressionEvaluationException(this,
          "operand is " + paTok + ":" + paTok.getClass().getName() + " ; variable expected");
    }

    Object valObject = paTok.eval(pRunEnv);

    if (valObject == null) {
      throw new ExpressionEvaluationException(this, "operand missed");
    }
    if (!(valObject instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + valObject + ":" + valObject.getClass().getName() + " ; numeric expected");
    }

    valObject = (new BigDecimal("" + valObject)).add(new BigDecimal(-1));

    // pRunEnv.setVariable(paTok.getSource(), valObject);
    Assignable varNameTok = (Assignable) paTok;

    varNameTok.set(pRunEnv, valObject);

    // return eval(valObject);
    return valObject;
  }// end method

  /**
   * process the operator
   */

  /*
   public final Object eval(final Object paObjValue) throws ExpressionEvaluationException {
   if (paObjValue == null) {
   throw new ExpressionEvaluationException(this, "operand missed");
   }
   if (!(paObjValue instanceof Number)) {
   throw new ExpressionEvaluationException(this,
   "left operand is " + paObjValue + ":" + paObjValue.getClass().getName() + " ; numeric expected");
   }

   return new BigDecimal("" + paObjValue).negate();
   }// end method eval
   */

}// end class
