package de.cnc.expression.tokencomplex;

import de.cnc.expression.AbstractToken;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionParseException;

/**
 * Member-Dot-Operator <b>&lt;index&gt;.member</b>  zur 
 * Adressierung eines Members eines Objekts . <br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: AbstractDotOperator.java,v 1.1 2005/04/12 10:09:37 hkuecker Exp $
 */
public abstract class AbstractDotOperator extends AbstractToken
{
    /**
     * der Ausdruck, hinter dem ein Dot-Operator steht
     */
    protected AbstractToken dottedExpression;
    
    /**
     * constructor
     */
    public AbstractDotOperator(
        String paStrOriginalSource,
        int paIntLine,
        int paIntCol)
    {
        super(paStrOriginalSource, paIntLine, paIntCol);
        // System.out.println( "constructor paStrOriginalSource:"+paStrOriginalSource);
    } // end constructor


    /**
     *
     */
    public static AbstractDotOperator parse(
            final String pStr, //--
            final int pIntLine, //--
            final int pIntCol, //--
            final String pStrOriginalSource) //--
        throws ExpressionParseException
    { // , NoSuchTokenException {
//System.out.println("AbstractDotOperator#parse: " + pStr);
        if (!pStr.startsWith(".")) {
            return null;
        }

        //erst suchen nach Methode wegen ffnender Klammer
        AbstractDotOperator dotElem = MethodDotOperator.parseMethodDotOperator(pStr, pIntLine, pIntCol, pStrOriginalSource);
        if (dotElem != null)
        {
            return dotElem;
        }
        
        //Member
        dotElem = MemberDotOperator.parseMemberDotOperator(pStr, pIntLine, pIntCol, pStrOriginalSource);
        return dotElem;

    } // end method parse

    /**
     * set the dotted expression.
     * Setzt den Ausdruck, der am Ende eine Index-Element-Operator []
     * trgt. Somit wird sozusagen der hinter dem indizierten Ausdruck stehende
     * Index-Operator vor den indizierten Ausdruck gesetzt.
     * Demzufolge wird erst der indzierte Ausdruck evaluiert und auf dessen Ergebnis
     * dann der Index-Operator angewendet.
     * 
     * @param token
     */
    public void setDottedExpression(AbstractToken pToken)
    {
        this.dottedExpression = pToken;
        //aufaddieren der Lnge des gedotteten Ausdruckes,
        //da gilt >xxx[ yyy ]< Lnge indizierter Ausdruck + Lnge Index-Operator
        this.iSrcLength += pToken.getSrcLength();
    }

    /**
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    public final Object[] getCheckReturnTypes(/*int[] paIntArrTypes*/) 
    throws ExpressionParseException
    {
        // spter noch Erweiterung auf getypte Variable
        // durch Erweiterung in AbstractRuntimeEnvironment
//        if (!(Types.isInOrUnknown(Types.RET_TYPE_LIST_OR_ARRAY, paIntArrTypes))) {
//            throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "List or Array type expected");
//          }

        return new Object[]{ Types.UNKNOWN_CLASS };
    } // end method getReturnTypes

}
