package de.cnc.expression.tokencomplex;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.Assignable;
import de.cnc.expression.Expression;
import de.cnc.expression.assignmentoperators.AbstractAssignmentOperator;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;

/**
 * Zuweisung &lt;var&gt; := &lt;expression&gt;
 */
public class Assignment extends AbstractToken
{

    /**
     * dieser Variable soll der Wert zugewiesen werden
     */
    //protected Variable variable;
    public AbstractToken destinationSubExpression;

    /**
     * der Zuweisungsoperator
     */
    public AbstractAssignmentOperator operator;

    /**
     * der Wert dieser <b>expression</b> wird der Variablen zugewiesen
     */
    public Expression expression;

    /**
     * parse
     * 
     * @param paStr
     * @param paIntLine
     * @param paIntCol
     * @param paStrOriginalSource
     * @return
     * @throws ExpressionParseException
     */
//    public static Assignment parse(
//        String paStr,
//        int paIntLine,
//        int paIntCol,
//        String paStrOriginalSource)
//        throws ExpressionParseException
//    {
//        //DebugUtil.printMethodInfo();
//        Assignment assmt =
//            new Assignment(paStrOriginalSource, paIntLine, paIntCol);
//
//        int iRun = 0;
//
//        // Einlesen der Variable der zuzuweisen ist
//        SubExpression destination =
//            Variable.parse(paStr, paIntLine, paIntCol, paStrOriginalSource);
//
//        if (varName == null)
//        {
//            return null; // keine Zuweisung erkannt
//        }
//
//        iRun += varName.getSrcLength();
//
//        //System.out.println( "varName:"+varName );
//
//        // danach null bis beliebig viele weisse zeichen
//        int iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
//
//        iRun += iWhites;
//
//        AbstractAssignmentOperator assOperator =
//            AbstractAssignmentOperator.parseAssignmentOperator(
//                paStr.substring(iRun),
//                paIntLine,
//                iRun + paIntCol,
//                paStrOriginalSource);
//
//        if (assOperator == null)
//        {
//            return null; // keine Zuweisung erkannt
//        }
//
//        iRun = assOperator.getSrcLength();
//
//        // danach null bis beliebig viele weisse zeichen
//        iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
//        iRun += iWhites;
//
//        Expression expr =
//            Expression.parseEasy(
//                paStr.substring(iRun),
//                paIntLine,
//                iRun + paIntCol,
//                paStrOriginalSource);
//
//        iRun += expr.getSrcLength();
//
//        if (expr == null)
//        {
//            // throw new ExpressionParseException(paIntLine, paIntCol + iRun, paStrOriginalSource, "expression expected");
//            return null;
//        }
//
//        // danach null bis beliebig viele weisse zeichen
//        if (iRun < paStr.length())
//        {
//            iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
//            iRun += iWhites;
//        }
//
//        assmt.variable = varName;
//        assmt.operator = assOperator;
//        assmt.expression = expr;
//        assmt.strSource = paStr.substring(0, iRun);
//        assmt.iSrcLength = iRun;
//
//        return assmt;
//
//    } // end method parse

    /**
     * constructor
     */
    public Assignment(String paStrOriginalSource, int paIntLine, int paIntCol)
    {
        super(paStrOriginalSource, paIntLine, paIntCol);
    } // end constructor

    /**
     * process the token
     */
    public final Object eval(final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionEvaluationException
    {
        final Object retValue; // = null;

        try
        {
            final Object setValue = this.expression.eval(pRunEnv);

            // this.variable.set(pRunEnv, oValue);

            // if (destinationSubExpression instanceof IndexOperator )
            //{
                Assignable assignable = (Assignable) this.destinationSubExpression;
                retValue = this.operator.eval(assignable, setValue, pRunEnv);
            //}

            // Rckgabe des gesetzten Wertes
            return retValue;
        }
        catch (ExpressionEvaluationException e)
        {
//System.err.println( e.getMessage() );
//e.printStackTrace();
//Log.fatal( e );
            throw e; // einfach weiterwerfen
        }
        catch (Exception e)
        {
//System.err.println( e.getMessage() );
//e.printStackTrace();
//Log.fatal( e );
            // Achtung: hier mehrfaches try-catch
            // was beim Auffangen einer ExpressionEvaluationException
            // if (e instanceof ExpressionEvaluationException)
            // {
            // throw e; // einfach weiterwerfen
            // }
            throw new ExpressionEvaluationException(
                getLine(),
                getCol(),
                getSource(),
                getOriginalSource(),
                "" + e);
        }
    } // end method eval

    /**
     * debug output
     */
    public String toString()
    {
        String strRet =
            getClass().getName()
                + " srcLength: "
                + getSrcLength()
                + " source: "
                + getSource()
                + "\n"
                + " destination: "
                + this.destinationSubExpression
                + " operator: "
                + this.operator
                + " expression: "
                + this.expression;

        return strRet;
    } // end method

    /**
     * rekursives Aufrufen der Klammerung der Ausdrcke
     * entsprechend der Operator-Prioritt.
     * Weiterleiten an den enthaltenen Ausdruck
     */
    public final void organizeOperatorPriority()
    {
        //fr eventuelle Index-Ausdrcke der Variablen
        //variable.organizeOperatorPriority();
        this.destinationSubExpression.organizeOperatorPriority();
        this.expression.organizeOperatorPriority();
    }

    /**
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    public final Object[] getCheckReturnTypes(//--
            final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionParseException
    {
        //return new Object[]{ Types.UNKNOWN_CLASS };
        this.destinationSubExpression.getCheckReturnTypes(pRunEnv);
        return this.expression.getCheckReturnTypes(pRunEnv);
    } // end method getReturnTypes

    /* (non-Javadoc)
     * @see de.cnc.expression.AbstractToken#isConst(de.cnc.expression.AbstractRuntimeEnvironment)
     */
    public final boolean isConst(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
    {
        return this.expression.isConst(pRunEnv);
    }

    /* (non-Javadoc)
     * @see de.cnc.expression.AbstractToken#getConstVal(de.cnc.expression.AbstractRuntimeEnvironment)
     */
    public final Object getConstVal(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
    {
        return this.expression.getConstVal(pRunEnv);
    }

} // end class
