package de.cnc.expression.tokencomplex;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.expression.infixoperators.AbstractInfixOperator;
import de.cnc.expression.prepostfixoperators.AbstractPrePostfixOperator;
import de.cnc.expression.tokensimple.ReservedWord;
import de.cnc.expression.util.ParserUtil;

/**
 * Klassen-Name mit Package.
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: Variable.java,v 1.1 2005/04/12 10:09:37 hkuecker Exp $
 */
public class ClassName extends AbstractToken
{

    /**
     * Lnge maximal 30 Zeichen
     * 
     * am 14.09.2004 umgestellt auf beliebig lange Bezeichner
     */
    //public final static int VAR_NAME_MAX_LEN = 30;

    /**
     * Name der Klasse
     */
    String className;

    /**
     *
     */
    public static ClassName parse( //--
        final String paStr, //--
        final int paIntLine, //--
        final int paIntCol, //--
        final String paStrOriginalSource)
        throws ExpressionParseException
    { // , NoSuchTokenException {
        ClassName className =
            new ClassName(paStrOriginalSource, paIntLine, paIntCol);

        // erste Stelle ein Buchstabe
        // danach Buchstabe oder Ziffer

        // Prfung auf Buchstabe am Anfang
        if (!Character.isJavaIdentifierStart(paStr.charAt(0)))
        {
            return null;
            // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no letter at start of variable name");
        }
        int iRun = 1;

        for (; /*iRun < VAR_NAME_MAX_LEN &&*/
            iRun < paStr.length(); iRun++)
        {
            char cTmp = paStr.charAt(iRun);

            if ( ! ( Character.isJavaIdentifierPart(cTmp) 
                    || ( cTmp == '.'  // Punkt im Variablen-Namen erlaubt
                        && iRun < paStr.length() - 1 //--
                        && Character.isJavaIdentifierStart(paStr.charAt( iRun + 1 ) ) )
                        ) )
            {
                // Ende des Klassen-Namens erreicht
                break;
            }
        }
        // Ende des Namens erreicht
        className.strSource = paStr.substring(0, iRun);
        className.className = paStr.substring(0, iRun);
        className.iSrcLength = iRun;
        // Variable-Name lnger als maximum
        //    if (iRun >= VAR_NAME_MAX_LEN) {
        //      throw new ExpressionParseException(paIntLine, paIntCol, paStr, "variable name to long");
        //    }

        // Prfen, ob Variablen-Name ein reserviertes Wort ist
        if (ReservedWord.isReservedWord(className.getSource()))
        {
            throw new ExpressionParseException(
                paIntLine,
                paIntCol,
                paStr,
                className.getSource() + " is an reserved word");
        }
        // Prfen, ob Variablen-Name ein Infix-Operator ist
        if (AbstractInfixOperator.isOperator(className.getSource()))
        {
            throw new ExpressionParseException(
                paIntLine,
                paIntCol,
                paStr,
                className.getSource() + " is an infix operator");
        }
        // Prfen, ob Variablen-Name ein Prefix-Operator ist
        if (AbstractPrePostfixOperator.isOperator(className.getSource()))
        {
            throw new ExpressionParseException(
                paIntLine,
                paIntCol,
                paStr,
                className.getSource() + " is an prefix operator");
        }

        // weiss-zeichen berspringen
        iRun += ParserUtil.parseWhitesAndComments(paStr.substring(iRun));


        className.strSource = paStr.substring(0, iRun);
        className.iSrcLength = iRun;

        return className;

    } // end method parse

    /**
     * constructor
     */
    public ClassName(
        String paStrOriginalSource,
        int paIntLine,
        int paIntCol)
    {
        super(paStrOriginalSource, paIntLine, paIntCol);
        // System.out.println( "constructor paStrOriginalSource:"+paStrOriginalSource);
    } // end constructor

    /**
     * process the token
     */
    public final Object eval(final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionEvaluationException
    {
        try
        {
            final Class varClass = Class.forName( this.className );
            return varClass;
        }
//        catch (ExpressionEvaluationException e)
//        {
////System.err.println( e.getMessage() );
////e.printStackTrace();
////Log.fatal( e );
//            throw e; // einfach weiterwerfen
//        }
        catch (Exception e)
        {
//System.err.println( e.getMessage() );
//e.printStackTrace();
//Log.fatal( e );
            // Achtung: hier mehrfaches try-catch
            // was beim Auffangen einer ExpressionEvaluationException
            // if (e instanceof ExpressionEvaluationException)
            // {
            // throw e; // einfach weiterwerfen
            // }
            throw new ExpressionEvaluationException(
                getLine(),
                getCol(),
                getSource(),
                getOriginalSource(),
                "" + e);
        }
    }


    /**
     * debug output
     */
    public String toString()
    {
        String strRet =
            getClass().getName()
                + " srcLength: "
                + getSrcLength()
                + " class name: "
                + getSource();

        return strRet;
    }

    /**
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    public final Object[] getCheckReturnTypes(//--
            final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionParseException
    {
        return new Object[]{ Class.class };
    } // end method getReturnTypes

    /**
     * rekursives Aufrufen der Klammerung der Ausdrcke
     * entsprechend der Operator-Prioritt.
     * Weiterleiten an den Index-Auisdruck ( var[ &lt;exp&gt; ] )
     */
    public final void organizeOperatorPriority()
    {
    }

    /**
     * set value unter beachtung array index
     */
    public void set(AbstractRuntimeEnvironment pRunEnv, Object paObjValue)
        throws ExpressionEvaluationException
    {
//        if (indexExpressionList == null || indexExpressionList.isEmpty())
//        {
//            // kein Index
//            pRunEnv.setVariable(strVarName, paObjValue);
//        }
//        else
//        {
//            // Holen Array oder List oder Map
//            Object objColl = pRunEnv.getVariable(strVarName); // Holen Array oder List oder Map
//
//            for (int i = 0; i < indexExpressionList.size(); i++)
//            {
//                // Index auf Array oder java.util.List
//                final Expression runIndexExpr = indexExpressionList.get( i );
//                Object inxObj = IndexOperator.evalIndex( runIndexExpr , pRunEnv );
//
//                if (objColl != null && objColl.getClass().isArray())
//                {
//                    // Array
//                    int iIndex = ((Number) inxObj).intValue();
//
//                    IndexOperator.checkArrayIndex(runIndexExpr, objColl, iIndex);
//
//                    Object objValue = paObjValue;
//
//                    if ( i < indexExpressionList.size() - 1 )
//                    {
//                        //nchster Index
//                        objColl = java.lang.reflect.Array.get(objColl, iIndex);
//                    }
//                    else
//                    {
//                        // ziemlich instabile Sache
//                        // das Konvertieren sollte gecatcht werden
//                        if (objColl instanceof boolean[])
//                        {
//                            objValue = Boolean.valueOf(objValue.toString());
//                        }
//                        else if (objColl instanceof byte[])
//                        {
//                            objValue = new Byte(objValue.toString());
//                        }
//                        else if (objColl instanceof char[])
//                        {
//                            objValue = new Character(objValue.toString().charAt(0));
//                        }
//                        else if (objColl instanceof double[])
//                        {
//                            objValue = new Double(objValue.toString());
//                        }
//                        else if (objColl instanceof float[])
//                        {
//                            objValue = new Float(objValue.toString());
//                        }
//                        else if (objColl instanceof int[])
//                        {
//                            // hier noch Behandlung von x.0 einbauen
//                            objValue = new Integer(objValue.toString());
//                        }
//                        else if (objColl instanceof long[])
//                        {
//                            objValue = new Long(objValue.toString());
//                        }
//                        else if (objColl instanceof short[])
//                        {
//                            objValue = new Short(objValue.toString());
//                        }
//
//                        if ( i < indexExpressionList.size() - 1 )
//                        {
//                            //nchster Index
//                            objColl = java.lang.reflect.Array.get(objColl, iIndex);
//                        }
//                        else
//                        {
//                            java.lang.reflect.Array.set(objColl, iIndex, objValue);
//                        }
//                    }
//                }
//                else if (objColl instanceof java.util.List)
//                {
//                    // java.util.List
//                    int iIndex = ((Number) inxObj).intValue();
//                    List listValue = (List) objColl;
//
//                    IndexOperator.checkListIndex( runIndexExpr, listValue , iIndex);
//
//                    if ( i < indexExpressionList.size() - 1 )
//                    {
//                        //nchster Index
//                        objColl = listValue.get(iIndex);
//                    }
//                    else
//                    {
//                        listValue.set(iIndex, paObjValue);
//                    }
//                }
//                else if (objColl instanceof java.util.Map)
//                {
//                    // java.util.Map
//                    Map mapValue = (Map) objColl;
//
//                    if ( i < indexExpressionList.size() - 1 )
//                    {
//                        //nchster Index
//                        objColl = mapValue.get(inxObj);
//                    }
//                    else
//                    {
//                        mapValue.put(inxObj, paObjValue);
//                    }
//                }
//                else
//                {
//                    throw new ExpressionEvaluationException(
//                        getLine(),
//                        getCol(),
//                        getSource(),
//                        getOriginalSource(),
//                        "variable is no array or java.util.List ");
//                }
//            }
//        }
            pRunEnv.setVariable(this.className, paObjValue);
            }

    /* (non-Javadoc)
     * @see de.cnc.expression.AbstractToken#isConst(de.cnc.expression.AbstractRuntimeEnvironment)
     */
    public final boolean isConst(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
    {
        return true;
    }

    /* (non-Javadoc)
     * @see de.cnc.expression.AbstractToken#getConstVal(de.cnc.expression.AbstractRuntimeEnvironment)
     */
    public final Object getConstVal(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
    {
        return eval(pRunEnv);
    }

} // end class
