package de.cnc.expression.tokencomplex;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.Assignable;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.expression.infixoperators.AbstractInfixOperator;
import de.cnc.expression.prepostfixoperators.AbstractPrePostfixOperator;
import de.cnc.expression.tokensimple.ReservedWord;
import de.cnc.expression.util.ParserUtil;
import de.cnc.util.Log;

/**
 * Variable
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: Variable.java,v 1.1 2005/04/12 10:09:37 hkuecker Exp $
 */
public class Variable extends AbstractToken implements Assignable
{

    /**
     * Lnge maximal 30 Zeichen
     * 
     * am 14.09.2004 umgestellt auf beliebig lange Bezeichner
     */
    //public final static int VAR_NAME_MAX_LEN = 30;

    /**
     * Name der Variablen
     */
    String varNameStr;

    /**
     *
     */
    public static Variable parse(
        final String paStr, //--
        final int paIntLine, //--
        final int paIntCol, //--
        final String paStrOriginalSource)
        throws ExpressionParseException
    { // , NoSuchTokenException {
        Variable varName =
            new Variable(paStrOriginalSource, paIntLine, paIntCol);

        // erste Stelle ein Buchstabe
        // danach Buchstabe oder Ziffer

        // Prfung auf Buchstabe am Anfang
        if (!Character.isJavaIdentifierStart(paStr.charAt(0)))
        {
            return null;
            // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no letter at start of variable name");
        }
        int iRun = 1;

        for (; /*iRun < VAR_NAME_MAX_LEN &&*/
            iRun < paStr.length(); iRun++)
        {
            char cTmp = paStr.charAt(iRun);

//            if (cTmp == '.' /*&& iRun < VAR_NAME_MAX_LEN*/
//                && iRun < paStr.length()
//                && Character.isJavaIdentifierStart(paStr.charAt(iRun + 1)))
//            {
//            }
//            else 
            if ( ! ( Character.isJavaIdentifierPart(cTmp) 
                    || ( cTmp == ':'  // Doppelpunkt im Variablen-Namen erlaubt
                        && iRun < paStr.length() - 1 //--
                        && Character.isJavaIdentifierStart(paStr.charAt( iRun + 1 ) ) )
                        ) )
            {
                // Ende des Variablen-Namens erreicht
                break;
            }
        }
        // Ende des Namens erreicht
        varName.strSource = paStr.substring(0, iRun);
        varName.varNameStr = paStr.substring(0, iRun);
        varName.iSrcLength = iRun;
        // Variable-Name lnger als maximum
        //    if (iRun >= VAR_NAME_MAX_LEN) {
        //      throw new ExpressionParseException(paIntLine, paIntCol, paStr, "variable name to long");
        //    }

        // Prfen, ob Variablen-Name ein reserviertes Wort ist
        if (ReservedWord.isReservedWord(varName.getSource()))
        {
            throw new ExpressionParseException(
                paIntLine,
                paIntCol,
                paStr,
                varName.getSource() + " is an reserved word");
        }
        // Prfen, ob Variablen-Name ein Infix-Operator ist
        if (AbstractInfixOperator.isOperator(varName.getSource()))
        {
            throw new ExpressionParseException(
                paIntLine,
                paIntCol,
                paStr,
                varName.getSource() + " is an infix operator");
        }
        // Prfen, ob Variablen-Name ein Prefix-Operator ist
        if (AbstractPrePostfixOperator.isOperator(varName.getSource()))
        {
            throw new ExpressionParseException(
                paIntLine,
                paIntCol,
                paStr,
                varName.getSource() + " is an prefix operator");
        }

        // weiss-zeichen berspringen
        iRun += ParserUtil.parseWhitesAndComments(paStr.substring(iRun));


        varName.strSource = paStr.substring(0, iRun);
        varName.iSrcLength = iRun;

        return varName;

    } // end method parse

    /**
     * constructor
     */
    public Variable(
        String paStrOriginalSource,
        int paIntLine,
        int paIntCol)
    {
        super(paStrOriginalSource, paIntLine, paIntCol);
        // System.out.println( "constructor paStrOriginalSource:"+paStrOriginalSource);
    } // end constructor

    /**
     * process the token
     */
    public final Object eval(final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionEvaluationException
    {
        // spter soll noch eine Unterscheidung zwischen null-value und nicht vorhanden erfolgen
        // jetzt wird erst mal einfach null fr nicht vorhanden zurckgegeben

        // System.out.println("strVarName:" + strVarName+":");

        Object objValue = pRunEnv.getVariable(this.varNameStr);

        //System.out.println("objValue:" + objValue);

        return objValue;
    }


    /**
     * debug output
     */
    public String toString()
    {
        String strRet =
            getClass().getName()
                + " srcLength: "
                + getSrcLength()
                + " variable name: "
                + getSource();

        return strRet;
    }

    /**
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    public final Object[] getCheckReturnTypes(//--
            final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionParseException
    {
        if ( pRunEnv == null )
        {
            return new Object[] { Types.UNKNOWN_CLASS };
        }
        try
        {
            // spter noch Erweiterung auf getypte Variable
            // durch Erweiterung in AbstractRuntimeEnvironment
            //return new int[] { Types.RET_TYPE_UNKNOWN };
            Class varClass = pRunEnv.getVariableType( this.varNameStr );
            
            if ( varClass == null )
            {
                return new Object[] { Types.UNKNOWN_CLASS };
            }
            return new Object[] { varClass };
        }
        catch (ExpressionEvaluationException e)
        {
            Log.fatal( e );
            throw new ExpressionParseException( //--
                    e.getLine(),//--
                    e.getCol(), //--
                    this.varNameStr, //--
                    e.getMessage());
        }
    } // end method getReturnTypes

    /**
     * rekursives Aufrufen der Klammerung der Ausdrcke
     * entsprechend der Operator-Prioritt.
     * Weiterleiten an den Index-Auisdruck ( var[ &lt;exp&gt; ] )
     */
    public final void organizeOperatorPriority()
    {
    }

    /**
     * set value unter beachtung array index
     */
    public void set(AbstractRuntimeEnvironment pRunEnv, Object paObjValue)
        throws ExpressionEvaluationException
    {
//        if (indexExpressionList == null || indexExpressionList.isEmpty())
//        {
//            // kein Index
//            pRunEnv.setVariable(strVarName, paObjValue);
//        }
//        else
//        {
//            // Holen Array oder List oder Map
//            Object objColl = pRunEnv.getVariable(strVarName); // Holen Array oder List oder Map
//
//            for (int i = 0; i < indexExpressionList.size(); i++)
//            {
//                // Index auf Array oder java.util.List
//                final Expression runIndexExpr = indexExpressionList.get( i );
//                Object inxObj = IndexOperator.evalIndex( runIndexExpr , pRunEnv );
//
//                if (objColl != null && objColl.getClass().isArray())
//                {
//                    // Array
//                    int iIndex = ((Number) inxObj).intValue();
//
//                    IndexOperator.checkArrayIndex(runIndexExpr, objColl, iIndex);
//
//                    Object objValue = paObjValue;
//
//                    if ( i < indexExpressionList.size() - 1 )
//                    {
//                        //nchster Index
//                        objColl = java.lang.reflect.Array.get(objColl, iIndex);
//                    }
//                    else
//                    {
//                        // ziemlich instabile Sache
//                        // das Konvertieren sollte gecatcht werden
//                        if (objColl instanceof boolean[])
//                        {
//                            objValue = Boolean.valueOf(objValue.toString());
//                        }
//                        else if (objColl instanceof byte[])
//                        {
//                            objValue = new Byte(objValue.toString());
//                        }
//                        else if (objColl instanceof char[])
//                        {
//                            objValue = new Character(objValue.toString().charAt(0));
//                        }
//                        else if (objColl instanceof double[])
//                        {
//                            objValue = new Double(objValue.toString());
//                        }
//                        else if (objColl instanceof float[])
//                        {
//                            objValue = new Float(objValue.toString());
//                        }
//                        else if (objColl instanceof int[])
//                        {
//                            // hier noch Behandlung von x.0 einbauen
//                            objValue = new Integer(objValue.toString());
//                        }
//                        else if (objColl instanceof long[])
//                        {
//                            objValue = new Long(objValue.toString());
//                        }
//                        else if (objColl instanceof short[])
//                        {
//                            objValue = new Short(objValue.toString());
//                        }
//
//                        if ( i < indexExpressionList.size() - 1 )
//                        {
//                            //nchster Index
//                            objColl = java.lang.reflect.Array.get(objColl, iIndex);
//                        }
//                        else
//                        {
//                            java.lang.reflect.Array.set(objColl, iIndex, objValue);
//                        }
//                    }
//                }
//                else if (objColl instanceof java.util.List)
//                {
//                    // java.util.List
//                    int iIndex = ((Number) inxObj).intValue();
//                    List listValue = (List) objColl;
//
//                    IndexOperator.checkListIndex( runIndexExpr, listValue , iIndex);
//
//                    if ( i < indexExpressionList.size() - 1 )
//                    {
//                        //nchster Index
//                        objColl = listValue.get(iIndex);
//                    }
//                    else
//                    {
//                        listValue.set(iIndex, paObjValue);
//                    }
//                }
//                else if (objColl instanceof java.util.Map)
//                {
//                    // java.util.Map
//                    Map mapValue = (Map) objColl;
//
//                    if ( i < indexExpressionList.size() - 1 )
//                    {
//                        //nchster Index
//                        objColl = mapValue.get(inxObj);
//                    }
//                    else
//                    {
//                        mapValue.put(inxObj, paObjValue);
//                    }
//                }
//                else
//                {
//                    throw new ExpressionEvaluationException(
//                        getLine(),
//                        getCol(),
//                        getSource(),
//                        getOriginalSource(),
//                        "variable is no array or java.util.List ");
//                }
//            }
//        }
            pRunEnv.setVariable(this.varNameStr, paObjValue);
            }

//    /**
//     * set for calling in SetVariableForNameFunction
//     */
//    public static void setVariable(
//        String paStrVarName,
//        Object paObjValue,
//        AbstractRuntimeEnvironment pRunEnv,
//        int paIntLine,
//        int paIntCol,
//        String paStrOriginalSource)
//        throws ExpressionEvaluationException
//    {
//        Variable varName = null;
//
//        try
//        {
//            varName =
//                Variable.parse(
//                    paStrVarName,
//                    paIntLine,
//                    paIntCol,
//                    paStrOriginalSource);
//            
////???Parsen IndexOperator???
//        
//            //hier organisieren der Operator-Prioritt, weil der Variable-Name mit eventuellen Indizes erst zur Laufzeit geparst wird
//            varName.organizeOperatorPriority();
//        }
//        catch (ExpressionParseException expParExc)
//        {
//            throw new ExpressionEvaluationException(
//                paIntLine,
//                paIntCol,
//                paStrVarName,
//                paStrOriginalSource,
//                expParExc.toString());
//        }
//        if (varName == null)
//        {
//            throw new ExpressionEvaluationException(
//                paIntLine,
//                paIntCol,
//                paStrVarName,
//                paStrOriginalSource,
//                "string " + paStrVarName + " is no variable name");
//        }
//
//        varName.set(pRunEnv, paObjValue);
//    } // end method

    /**
     * remove variable unter beachtung array index
     */
    public Object remove(AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionEvaluationException
    {

//        if (indexExpressionList == null || indexExpressionList.isEmpty())
//        {
//            // kein Index
//            pRunEnv.removeVariable(strVarName);
//        }
//        else
//        {
//            // Holen Array oder List oder Map
//            Object objColl = pRunEnv.getVariable(strVarName);
//
//            for (int i = 0; i < indexExpressionList.size(); i++)
//            {
//                // Index auf Array oder java.util.List
//                final Expression runIndexExpr = indexExpressionList.get( i );
//                Object inxObj = IndexOperator.evalIndex( runIndexExpr , pRunEnv );
//
//                if (objColl != null && objColl.getClass().isArray())
//                {
//                    // Array
//                    int iIndex = ((Number) inxObj).intValue();
//
//                    IndexOperator.checkArrayIndex( runIndexExpr, objColl, iIndex);
//
//                    if ( i < indexExpressionList.size() - 1 )
//                    {
//                        //nchster Index
//                        objColl = java.lang.reflect.Array.get(objColl, iIndex);
//                    }
//                    else
//                    {
//                        java.lang.reflect.Array.set(objColl, iIndex, null);
//                    }
//
//                }
//                else if (objColl instanceof java.util.List)
//                {
//                    // java.util.List
//                    int iIndex = ((Number) inxObj).intValue();
//                    List listValue = (List) objColl;
//
//                    IndexOperator.checkListIndex(runIndexExpr , listValue, iIndex);
//
//                    if ( i < indexExpressionList.size() - 1 )
//                    {
//                        //nchster Index
//                        objColl = listValue.get(iIndex);
//                    }
//                    else
//                    {
//                        listValue.remove(iIndex);
//                    }
//                }
//                else if (objColl instanceof java.util.Map)
//                {
//                    // java.util.Map
//                    Map mapValue = (Map) objColl;
//
//                    if ( i < indexExpressionList.size() - 1 )
//                    {
//                        //nchster Index
//                        objColl = mapValue.get(inxObj);
//                    }
//                    else
//                    {
//                        mapValue.remove(inxObj);
//                    }
//                }
//                else
//                {
//                    throw new ExpressionEvaluationException(
//                        getLine(),
//                        getCol(),
//                        getSource(),
//                        getOriginalSource(),
//                        "variable is no array or java.util.List or java.util.Map");
//                }
//            }
//
//        }
        return pRunEnv.removeVariable(this.varNameStr);
    } // end method

//    /**
//     * remove variable for calling in GetPropertyFunction
//     */
//    private static void removeVariable(
//        String paStrVarName,
//        AbstractRuntimeEnvironment pRunEnv,
//        int paIntLine,
//        int paIntCol,
//        String paStrOriginalSource)
//        throws ExpressionEvaluationException
//    {
//        Variable variable = null;
//
//        try
//        {
//            variable =
//                Variable.parse(
//                    paStrVarName,
//                    paIntLine,
//                    paIntCol,
//                    paStrOriginalSource);
//        }
//        catch (ExpressionParseException expParExc)
//        {
//            throw new ExpressionEvaluationException(
//                paIntLine,
//                paIntCol,
//                paStrVarName,
//                paStrOriginalSource,
//                expParExc.toString());
//        }
//        if (variable == null)
//        {
//            throw new ExpressionEvaluationException(
//                paIntLine,
//                paIntCol,
//                paStrVarName,
//                paStrOriginalSource,
//                "string " + paStrVarName + " is no variable name");
//        }
//
//        variable.remove(pRunEnv);
//    } // end method

    /* (non-Javadoc)
     * @see de.cnc.expression.AbstractToken#isConst(de.cnc.expression.AbstractRuntimeEnvironment)
     */
    public boolean isConst(AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
    {
        // TODO Auto-generated method stub
        return false;
    }

    /* (non-Javadoc)
     * @see de.cnc.expression.AbstractToken#getConstVal(de.cnc.expression.AbstractRuntimeEnvironment)
     */
    public Object getConstVal(AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
    {
        // TODO Auto-generated method stub
        return null;
    }

} // end class
