package de.cnc.expression.tokencomplex;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.expression.util.ParserUtil;
import de.cnc.util.Log;

/**
 * Variablen-Deklaration var &lt;varName&gt; &lt;varClass&gt;
 */
public class VariableTypeDeclaration extends AbstractToken
{

    /**
     * Die deklarierte Variable
     */
    protected Variable variable;

    /**
     * Type der Variablen.
     */
    public ClassName className;

    /**
    *
    */
   public static VariableTypeDeclaration parse(
       final String pStr, //--
       final int pIntLine, //--
       final int pIntCol, //--
       final String pStrOriginalSource)
       throws ExpressionParseException
   { // , NoSuchTokenException {
       
       if ( pStr.length() < "var".length() + 4 )
       {
//System.out.println("return null1");           
           return null;
       }
       if ( ! pStr.startsWith( "var" ) )
       {
//System.out.println("return null2");           
           return null;
       }
       if ( ! Character.isWhitespace( pStr.charAt( "var".length() ) ) )
       {
//System.out.println("return null3");           
           return null;
       }

       VariableTypeDeclaration varTypDcl =
           new VariableTypeDeclaration(pStrOriginalSource, pIntLine, pIntCol);

       int iRun = "var".length() + 1;

       // weiss-zeichen berspringen
       iRun += ParserUtil.parseWhitesAndComments(pStr.substring(iRun));

       ClassName className = ClassName.parse( //--
               pStr.substring( iRun ), //--
               pIntLine, //--
               pIntCol, //--
               pStrOriginalSource);

       if ( className == null )
       {
//System.out.println("return null5");           
           //return null;
           
           throw new ExpressionParseException(
                   pIntLine,
                   pIntCol,
                   pStr,
                   "class name expected");
       }
       
       varTypDcl.className = className;
       
       iRun += className.getSrcLength();
       
       // weiss-zeichen berspringen
       iRun += ParserUtil.parseWhitesAndComments(pStr.substring(iRun));

       Variable variable = Variable.parse( //--
               pStr.substring( iRun ), //--
               pIntLine, //--
               pIntCol, //--
               pStrOriginalSource);
       
       if ( variable == null )
       {
//System.out.println("return null4");           
           //return null;

           throw new ExpressionParseException(
                   pIntLine,
                   pIntCol,
                   pStr,
                   "variable name expected");

       }
       
       varTypDcl.variable = variable;
       
       iRun += variable.getSrcLength();
       
       // weiss-zeichen berspringen
       iRun += ParserUtil.parseWhitesAndComments(pStr.substring(iRun));

       varTypDcl.strSource = pStr.substring(0, iRun);
       varTypDcl.iSrcLength = iRun;

       return varTypDcl;

   } // end method parse

    /**
     * constructor
     */
    public VariableTypeDeclaration(String paStrOriginalSource, int paIntLine, int paIntCol)
    {
        super(paStrOriginalSource, paIntLine, paIntCol);
    } // end constructor

    /**
     * process the token
     */
    public final Object eval(final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionEvaluationException
    {
        try
        {
            final String varNameStr = this.variable.getSource();
            
            final Class varClass = (Class) this.className.eval(pRunEnv);
            
            pRunEnv.setVariableType( varNameStr, varClass );
            
            // Rckgabe des gesetzten Wertes
            return varClass;
        }
        catch (ExpressionEvaluationException e)
        {
//System.err.println( e.getMessage() );
//e.printStackTrace();
//Log.fatal( e );
            throw e; // einfach weiterwerfen
        }
        catch (Exception e)
        {
//System.err.println( e.getMessage() );
//e.printStackTrace();
Log.fatal( e );
            // Achtung: hier mehrfaches try-catch
            // was beim Auffangen einer ExpressionEvaluationException
            // if (e instanceof ExpressionEvaluationException)
            // {
            // throw e; // einfach weiterwerfen
            // }
            throw new ExpressionEvaluationException(
                getLine(),
                getCol(),
                getSource(),
                getOriginalSource(),
                "" + e);
        }
    } // end method eval

    /**
     * debug output
     */
    public String toString()
    {
        String strRet =
            getClass().getName()
                + " srcLength: "
                + getSrcLength()
                + " source: "
                + getSource()
                + "\n"
                + " variable: "
                + this.variable.getSource()
                + " className: "
                + this.className.getSource();

        return strRet;
    } // end method

    /**
     * rekursives Aufrufen der Klammerung der Ausdrcke
     * entsprechend der Operator-Prioritt.
     * Weiterleiten an den enthaltenen Ausdruck
     */
    public final void organizeOperatorPriority()
    {
        ; // nichts zu tun
    }

    /**
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    public final Object[] getCheckReturnTypes(//--
            final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionParseException
    {
        try
        {
            // hier schon ausfhren fr Typ-Prfung
            eval(pRunEnv);
        }
        catch (ExpressionEvaluationException e)
        {
            throw new ExpressionParseException(//--
                    getLine(),
                    getCol(),
                    getSource(),
                    e.getMessage());
        }
        
        //TODO
        // spter noch Erweiterung auf getypte Variable
        // durch Erweiterung in AbstractRuntimeEnvironment
        //return new int[] { Types.RET_TYPE_UNKNOWN };
        return new Object[] { Class.class };
    } // end method getReturnTypes

    /* (non-Javadoc)
     * @see de.cnc.expression.AbstractToken#isConst(de.cnc.expression.AbstractRuntimeEnvironment)
     */
    public boolean isConst(AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
    {
        return true;
    }

    /* (non-Javadoc)
     * @see de.cnc.expression.AbstractToken#getConstVal(de.cnc.expression.AbstractRuntimeEnvironment)
     */
    public Object getConstVal(AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
    {
        return eval(pRunEnv);
    }

    
} // end class
