package de.cnc.expression.tokensimple;


import java.sql.Time;
import java.util.Date;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.util.StringUtil;


public class DateLiteral extends AbstractToken {

    Time time;
    
  public static DateLiteral parse(//--
          final String paStr, //--
          final int paIntLine, //--
          final int paIntCol, //--
          final String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    DateLiteral timeLit = new DateLiteral(paStrOriginalSource, paIntLine, paIntCol);

    // Prfung auf Ziffer am Anfang
    if (!Character.isDigit(paStr.charAt(0))) {
      return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no digit at start of number");
    }
    int iRun = 0;

    //Stunden-Angabe
    String hourStr = "" + paStr.charAt(iRun);
    iRun++;
    if (iRun >= paStr.length() )
    {
        return null;
    }
    //zweite Stelle der Stunden-Angabe
    char cTmp = paStr.charAt(iRun);
    if ( Character.isDigit(cTmp) )
    {
        hourStr += cTmp;
        iRun++;
    }
    if (iRun >= paStr.length() )
    {
        return null;
    }
    
    //Doppelpunkt zwischen Stunde und Minuten
    cTmp = paStr.charAt(iRun);
    if (cTmp != ':')
    {
        //kein Time-Literal
        return null;
    }
    iRun++;
        
    if (iRun >= paStr.length() )
    {
        throw new ExpressionParseException(paIntLine, iRun, paStr, "minutes digit for time literal expected");
    }
    
    //Minuten-Angabe
    cTmp = paStr.charAt(iRun);
    if ( ! Character.isDigit( cTmp )){
        throw new ExpressionParseException(paIntLine, iRun, paStr, "digit for time literal expected");
    }
    String minutesStr = "" + cTmp;

    iRun++;
    
    //zweite Stelle der Stunden-Angabe
    if (iRun < paStr.length() )
    {
        cTmp = paStr.charAt(iRun);
        if ( Character.isDigit(cTmp) )
        {
            minutesStr += cTmp;
            iRun++;
        }
    }

    //Sekunden
    String secondsStr = "0";
    String secondFractionStr = "0";
    
    //Doppelpunkt zwischen Minuten und Sekunden
    if (iRun < paStr.length() )
    {
        cTmp = paStr.charAt(iRun);
        if (cTmp == ':')
        {
            //es gibt Sekunden
            secondsStr = "";
            iRun++;
            while ( iRun < paStr.length() //--
                    && Character.isDigit( paStr.charAt(iRun) ) ){
                secondsStr += paStr.charAt(iRun);
                iRun++;
//System.out.println( "1secondsStr: >" + secondsStr + "<" );    
            }
//if (iRun < paStr.length()){
//System.out.println( "2secondsStr: >" + secondsStr + "<" );    
//System.out.println( "paStr.charAt(iRun): " + paStr.charAt(iRun) );
//}
            if ( iRun < paStr.length() //--
                    && paStr.charAt(iRun) == '.' ){
                //es gibt Kommastellen bei den Sekunden
                //gebrochene Sekunden realisieren
                //secondsStr += paStr.charAt(iRun);
                iRun++;
                secondFractionStr ="";
                while ( iRun < paStr.length() //--
                        && Character.isDigit( paStr.charAt(iRun) ) ){
                    //secondsStr += paStr.charAt(iRun);
                    secondFractionStr += paStr.charAt(iRun);
                    iRun++;
//System.out.println( "3secondsStr: >" + secondsStr + "<" );    
                }
            }
        }
    }
    
//System.out.println( "4secondsStr: >" + secondsStr + "<" );    

    int hourInt = StringUtil.str2int( hourStr );
    if (hourInt > 23)
    {
        throw new ExpressionParseException(paIntLine, iRun, paStr, "hour maximum is 23 for for time literal");
    }
    
    int minutesInt = StringUtil.str2int( minutesStr );
    if (minutesInt > 59)
    {
        throw new ExpressionParseException(paIntLine, iRun, paStr, "minute maximum is 59 for for time literal");
    }

    int secondsInt = StringUtil.str2int( secondsStr );
    if (secondsInt >= 60)
    {
        throw new ExpressionParseException(paIntLine, iRun, paStr, "second maximum is 59.999999 for for time literal");
    }
    double secondFractionDbl = StringUtil.str2double( "0." + secondFractionStr );

    // Ende des Time-Literals erreicht
    timeLit.strSource = paStr.substring(0, iRun);
    timeLit.iSrcLength = iRun;

    //System.out.println( "" + hourInt + " " + minutesInt + " " + secondsInt);
    
    //long millis = Math.round( ( ( ( ( ( hourInt - 1 ) * 60 ) + minutesInt ) * 60 ) + secondsInt ) * 1000 );
    //timeLit.time = new Time( millis );
    timeLit.time = new Time( hourInt , minutesInt , secondsInt );

    //gebrochene Sekunden aufrechnen
    //long millis = timeLit.time.getTime();
    //long fractMillis = Math.round( secondFractionDbl * 1000 );
    //long entireMillis = millis + fractMillis;
    //timeLit.time = new Time( entireMillis );
    timeLit.time = addFractionalMillis( timeLit.time , secondFractionDbl );
    
//    try {
//      timeLit.bdValue = new BigDecimal(paStr.substring(0, iRun));
//      // System.out.println("NumberLiteral "+ paStr.substring(0, iRun) + " " + numLit.bdValue ) ;
//    } catch (NumberFormatException nfex) {
//      // throw new RuntimeException("Programmfehler");
//      return null; // scheinbar doch keine Zahl
//    }

    return timeLit;

  }// end method parse

  /**
   * Auf-Addieren eines gebrochenen
   * Sekundenanteils (0 ... 0.999) 
   * auf die spezifizierte {@link Time},
   * weil der Konstruktor von {@link Time}
   * keine Angabe gebrochener Sekunden
   * erlaubt.
   * 
   * @param pIntegralTime
   * @param pFractionalMillis
   * @return
   */
  public static Time addFractionalMillis(//--
        final Time pIntegralTime, //--
        final double pFractionalMillis ){
    long millis = pIntegralTime.getTime();
    long fractMillis = Math.round( pFractionalMillis * 1000 );
    long entireMillis = millis + fractMillis;
    return new Time( entireMillis );
  }
  
  /**
   * constructor
   */
  public DateLiteral(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    return this.time;
  }

  public String toString() {
    String strRet = getClass().getName() + " srcLength: " + getSrcLength() + " value: " + this.time;

    return strRet;
  }

  /**
   * rekursives Aufrufen der Klammerung der Ausdrcke
   * entsprechend der Operator-Prioritt.
   * Nichts zu tun.
   */
  public final void organizeOperatorPriority()
  {
      ;
  }

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    return new Object[]{ Date.class };
  }// end method getReturnTypes

  /* (non-Javadoc)
   * @see de.cnc.expression.AbstractToken#isConst(de.cnc.expression.AbstractRuntimeEnvironment)
   */
  public final boolean isConst(AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
  {
      return true;
  }

  /* (non-Javadoc)
   * @see de.cnc.expression.AbstractToken#getConstVal(de.cnc.expression.AbstractRuntimeEnvironment)
   */
  public final Object getConstVal(AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
  {
      return this.time;
  }

}// end class
