package de.cnc.expression.tokensimple;

import java.math.BigDecimal;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.expression.util.DayOfWeek;
import de.cnc.expression.util.Month;

/**
 * Reserved Words true, false, null and so on
 */
public class ReservedWord extends AbstractToken {

  protected Object oValue;
  protected Object iRetType;

  public static ReservedWord parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    ReservedWord resWrd = new ReservedWord(paStrOriginalSource, paIntLine, paIntCol);

    if (isWord(paStr,"true",resWrd)) {
      resWrd.oValue = Boolean.TRUE;
      resWrd.iRetType = Boolean.class; // Types.RET_TYPE_BOOL;
      return resWrd;
    }
    if (isWord(paStr,"false",resWrd)) {
      resWrd.oValue = Boolean.FALSE;
      resWrd.iRetType = Boolean.class; // Types.RET_TYPE_BOOL;
      return resWrd;
    }
    if (isWord(paStr,"null",resWrd)) {
      resWrd.oValue = null;
      resWrd.iRetType = Types.NULL_CLASS;
      return resWrd;
    }
    if (isWord(paStr,"apostroph",resWrd)) {
      resWrd.oValue = "\"";
      resWrd.iRetType = String.class; //Types.RET_TYPE_STRING;
      return resWrd;
    }
    if (isWord(paStr,"quote",resWrd)) {
      resWrd.oValue = "\"";
      resWrd.iRetType = String.class; //Types.RET_TYPE_STRING;
      return resWrd;
    }
    if (isWord(paStr,"highcomma",resWrd)) {
      resWrd.oValue = "'";
      resWrd.iRetType = String.class; //Types.RET_TYPE_STRING;
      return resWrd;
    }
    if (isWord(paStr,"highComma",resWrd)) {
      resWrd.oValue = "'";
      resWrd.iRetType = String.class; //Types.RET_TYPE_STRING;
      return resWrd;
    }
    if (isWord(paStr,"newline",resWrd)) {
      resWrd.oValue = "\n";
      resWrd.iRetType = String.class; //Types.RET_TYPE_STRING;
      return resWrd;
    }
    if (isWord(paStr,"newLine",resWrd)) {
      resWrd.oValue = "\n";
      resWrd.iRetType = String.class; //Types.RET_TYPE_STRING;
      return resWrd;
    }
    if (isWord(paStr,"pi",resWrd)) {
      resWrd.oValue = new BigDecimal( Math.PI ) ;
      resWrd.iRetType = Number.class; //Types.RET_TYPE_NUM;
      return resWrd;
    }
    if (isWord(paStr,"PI",resWrd)) {
      resWrd.oValue = new BigDecimal( Math.PI ) ;
      resWrd.iRetType = Number.class; //Types.RET_TYPE_NUM;
      return resWrd;
    }
    if (isWord(paStr,"euler",resWrd)) {
      resWrd.oValue = new BigDecimal( Math.E ) ;
      resWrd.iRetType = Number.class; //Types.RET_TYPE_NUM;
      return resWrd;
    }
    if (isWord(paStr,"Euler",resWrd)) {
      resWrd.oValue = new BigDecimal( Math.E ) ;
      resWrd.iRetType = Number.class; //Types.RET_TYPE_NUM;
      return resWrd;
    }
    if (isWord(paStr,"EULER",resWrd)) {
      resWrd.oValue = new BigDecimal( Math.E ) ;
      resWrd.iRetType = Number.class; //Types.RET_TYPE_NUM;
      return resWrd;
    }
    // --- neue reservierte Wrter fr die Wochentage ---
    if (isWord(paStr,"SUNDAY",resWrd)) {
        resWrd.oValue = DayOfWeek.SUNDAY;
        resWrd.iRetType = DayOfWeek.class; // Types.RET_TYPE_DAY_OF_WEEK;
        return resWrd;
    }
    if (isWord(paStr,"MONDAY",resWrd)) {
        resWrd.oValue = DayOfWeek.MONDAY;
        resWrd.iRetType = DayOfWeek.class; // Types.RET_TYPE_DAY_OF_WEEK;
        return resWrd;
    }
    if (isWord(paStr,"TUESDAY",resWrd)) {
        resWrd.oValue = DayOfWeek.TUESDAY;
        resWrd.iRetType = DayOfWeek.class; // Types.RET_TYPE_DAY_OF_WEEK;
        return resWrd;
    }
    if (isWord(paStr,"WEDNESDAY",resWrd)) {
        resWrd.oValue = DayOfWeek.WEDNESDAY;
        resWrd.iRetType = DayOfWeek.class; // Types.RET_TYPE_DAY_OF_WEEK;
        return resWrd;
    }
    if (isWord(paStr,"THURSDAY",resWrd)) {
        resWrd.oValue = DayOfWeek.THURSDAY;
        resWrd.iRetType = DayOfWeek.class; // Types.RET_TYPE_DAY_OF_WEEK;
        return resWrd;
    }
    if (isWord(paStr,"FRIDAY",resWrd)) {
        resWrd.oValue = DayOfWeek.FRIDAY;
        resWrd.iRetType = DayOfWeek.class; // Types.RET_TYPE_DAY_OF_WEEK;
        return resWrd;
    }
    if (isWord(paStr,"SATURDAY",resWrd)) {
        resWrd.oValue = DayOfWeek.SATURDAY;
        resWrd.iRetType = DayOfWeek.class; // Types.RET_TYPE_DAY_OF_WEEK;
        return resWrd;
    }
    // --- neue reservierte Wrter fr die Monate ---
    if (isWord(paStr,"JANUARY",resWrd)) {
        resWrd.oValue = Month.JANUARY;
        resWrd.iRetType = Month.class; // Types.RET_TYPE_MONTH;
        return resWrd;
    }
    if (isWord(paStr,"FEBRUARY",resWrd)) {
        resWrd.oValue = Month.FEBRUARY;
        resWrd.iRetType = Month.class; // Types.RET_TYPE_MONTH;
        return resWrd;
    }
    if (isWord(paStr,"MARCH",resWrd)) {
        resWrd.oValue = Month.MARCH;
        resWrd.iRetType = Month.class; // Types.RET_TYPE_MONTH;
        return resWrd;
    }
    if (isWord(paStr,"APRIL",resWrd)) {
        resWrd.oValue = Month.APRIL;
        resWrd.iRetType = Month.class; // Types.RET_TYPE_MONTH;
        return resWrd;
    }
    if (isWord(paStr,"MAY",resWrd)) {
        resWrd.oValue = Month.MAY;
        resWrd.iRetType = Month.class; // Types.RET_TYPE_MONTH;
        return resWrd;
    }
    if (isWord(paStr,"JUNE",resWrd)) {
        resWrd.oValue = Month.JUNE;
        resWrd.iRetType = Month.class; // Types.RET_TYPE_MONTH;
        return resWrd;
    }
    if (isWord(paStr,"JULY",resWrd)) {
        resWrd.oValue = Month.JULY;
        resWrd.iRetType = Month.class; // Types.RET_TYPE_MONTH;
        return resWrd;
    }
    if (isWord(paStr,"AUGUST",resWrd)) {
        resWrd.oValue = Month.AUGUST;
        resWrd.iRetType = Month.class; // Types.RET_TYPE_MONTH;
        return resWrd;
    }
    if (isWord(paStr,"SEPTEMBER",resWrd)) {
        resWrd.oValue = Month.SEPTEMBER;
        resWrd.iRetType = Month.class; // Types.RET_TYPE_MONTH;
        return resWrd;
    }
    if (isWord(paStr,"OCTOBER",resWrd)) {
        resWrd.oValue = Month.OCTOBER;
        resWrd.iRetType = Month.class; // Types.RET_TYPE_MONTH;
        return resWrd;
    }
    if (isWord(paStr,"NOVEMBER",resWrd)) {
        resWrd.oValue = Month.NOVEMBER;
        resWrd.iRetType = Month.class; // Types.RET_TYPE_MONTH;
        return resWrd;
    }
    if (isWord(paStr,"DECEMBER",resWrd)) {
        resWrd.oValue = Month.DECEMBER;
        resWrd.iRetType = Month.class; // Types.RET_TYPE_MONTH;
        return resWrd;
    }
    
    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no digit at start of number");
  }// end method parse

  /**
   * Bestimmung ob das Wort passt mit
   * Seiteneffekt auf das bergebene
   * ReservedWord-Objekt.
   * 
   * @param pSrcStr
   * @return
   */
  private static boolean isWord( //--
          final String pSrcStr, //--
          final String pWordName, //--
          final ReservedWord pResWrd) {
      final boolean bRet = pWordName.equals(pSrcStr) || // ...
          ((pSrcStr.length() > pWordName.length() //--
                  && pSrcStr.startsWith(pWordName) //--
                  && !Character.isLetterOrDigit(pSrcStr.charAt( pWordName.length() ))));
      pResWrd.strSource = pWordName;
      pResWrd.iSrcLength = pWordName.length();
    return bRet;
  }

  /**
   * constructor
   */
  public ReservedWord(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public final Object eval(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException {
    return this.oValue;
  }

  /**
   * Prfroutine fr Variablen-Namen zur Vermeidung der
   * Benutzung reservierter Worte fr die Variablen-Namen. <br>
   * @see de.cnc.expression.tokencomplex.Variable#parse
   */
  public static boolean isReservedWord(String pStr) {
    return // ...
        "var".equals(pStr)  || // Variablen-Typ-Deklaration
        "new".equals(pStr)  || // new-Operator
        "true".equals(pStr)  || // ...
        "false".equals(pStr) || // ...
        "null".equals(pStr)  ||// ...
        "new".equals(pStr)   ||// ...
        "apostroph".equals(pStr) || // ...
        "quote".equals(pStr) || // ...
        "highcomma".equals(pStr) || // ...
        "highComma".equals(pStr) || // ...
        "newline".equals(pStr) || // ...
        "newLine".equals(pStr) ||  // ...
        "pi".equals(pStr) ||  // ...
        "PI".equals(pStr) ||  // ...
        "euler".equals(pStr) ||  // ...
        "Euler".equals(pStr) ||  // ...
        "EULER".equals(pStr) || // ...
        "SUNDAY"    .equals(pStr) || // ...
        "MONDAY"    .equals(pStr) || // ...
        "TUESDAY"   .equals(pStr) || // ...
        "WEDNESDAY" .equals(pStr) || // ...
        "THURSDAY"  .equals(pStr) || // ...
        "FRIDAY"    .equals(pStr) || // ...
        "SATURDAY"  .equals(pStr) || // ...
        "JANUARY"  .equals(pStr) || // ...
        "FEBRUARY"  .equals(pStr) || // ...
        "MARCH"  .equals(pStr) || // ...
        "APRIL"  .equals(pStr) || // ...
        "MAY"  .equals(pStr) || // ...
        "JUNE"  .equals(pStr) || // ...
        "JULY"  .equals(pStr) || // ...
        "AUGUST"  .equals(pStr) || // ...
        "SEPTEMBER"  .equals(pStr) || // ...
        "OCTOBER"  .equals(pStr) || // ...
        "NOVEMBER"  .equals(pStr) || // ...
        "DECEMBER"  .equals(pStr)    // ...
        ;
  }

  public final String toString() {
    String strRet = getClass().getName() + " srcLength: " + getSrcLength() + " value: " + this.oValue;

    return strRet;
  }

  /**
   * rekursives Aufrufen der Klammerung der Ausdrcke
   * entsprechend der Operator-Prioritt.
   * Nichts zu tun.
   */
  public final void organizeOperatorPriority()
  {
      ;
  }

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public final Object[] getCheckReturnTypes(//--
          final AbstractRuntimeEnvironment pRunEnv)
      throws ExpressionParseException
  {
    return new Object[] { this.iRetType };
  }// end method getReturnTypes

  /* (non-Javadoc)
   * @see de.cnc.expression.AbstractToken#isConst(de.cnc.expression.AbstractRuntimeEnvironment)
   */
  public boolean isConst(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
  {
      return true;
  }

  /* (non-Javadoc)
   * @see de.cnc.expression.AbstractToken#getConstVal(de.cnc.expression.AbstractRuntimeEnvironment)
   */
  public Object getConstVal(final AbstractRuntimeEnvironment pRunEnv) throws ExpressionEvaluationException
  {
      return this.oValue;
  }

}// end class
