package de.cnc.expression.util;

import java.sql.Time;
import java.util.Date;

import de.cnc.expression.functions.TimeOfDayFunction;

/**
 * Hilfsmethoden zum Vergleich von Objekten . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 */
public final class DateTimeCompareUtil {

    /**
     * compareTo-Methode mit Abfangung null.
     * Spezial fr Date und Time in der Expression-Engine.
     */
    public static final int dateTimeCompareTo(Comparable comPa1, Comparable comPa2) {
//  Log.debug( "CompareUtil#dateTimeCompareTo >" + comPa1 + "< with >" + comPa2 + "<" );
        
      if (comPa1 == null && comPa2 == null) {
        // wenn comPa1 und comPa1 null ist, dann gilt gleich
        return 0;
      } else if (comPa1 == null) {
        // wenn comPa1 null ist, dann gilt kleiner als
        return -1;
      } else if (comPa2 == null) {
        // wenn comPa2 null ist, dann gilt groesser als
        return 1;
      } else {

          //Sonderlocke Monat
          if (comPa1 instanceof Month || comPa2 instanceof Month){
              final Month com1Mon = Month.getInstance(comPa1);
              final Month com2Mon = Month.getInstance(comPa2);
              return com1Mon.getValue() - com2Mon.getValue();
          }
          
          //Sonderlocke Wochentag
          else if (comPa1 instanceof DayOfWeek || comPa2 instanceof DayOfWeek){
              final DayOfWeek com1Day = DayOfWeek.getInstance(comPa1);
              final DayOfWeek com2Day = DayOfWeek.getInstance(comPa2);
              return com1Day.getValue() - com2Day.getValue();
          }
          
          //Sonderlocke Uhrzeit innerhalb des Tages
          else if (comPa1 instanceof Time || comPa2 instanceof Time){
              final Time com1Time = TimeOfDayFunction.dateToTime( (Date) comPa1 );
              final Time com2Time = TimeOfDayFunction.dateToTime( (Date) comPa2 );
              return dateTimeCompareTo( new Long( com1Time.getTime() ) , new Long( com2Time.getTime() ) );
          }

          try {
            return comPa1.compareTo(comPa2);
        } catch (ClassCastException c) {
            String classStr1 = comPa1 == null ? "null" : comPa1.getClass().getName() ;
            String classStr2 = comPa2 == null ? "null" : comPa2.getClass().getName() ;
            throw new ClassCastException( "compare " + classStr1 + " with " + classStr2 );
        }
      }// end if
    }// end method dateTimeCompareTo

    public static final boolean lesser(Comparable comPa1, Comparable comPa2) {
        return (dateTimeCompareTo(comPa1, comPa2) < 0);
      }// end method less

      public static final boolean lesserEqual(Comparable comPa1, Comparable comPa2) {
        return (dateTimeCompareTo(comPa1, comPa2) <= 0);
      }// end method lessEqual

      public static final boolean greater(Comparable comPa1, Comparable comPa2) {
        return (dateTimeCompareTo(comPa1, comPa2) > 0);
      }// end method greater

      public static final boolean greaterEqual(Comparable comPa1, Comparable comPa2) {
        return (dateTimeCompareTo(comPa1, comPa2) >= 0);
      }// end method greaterEqual

      public static final boolean inRange(Comparable comIn, Comparable comFrom, Comparable comTo) {
          return ( greaterEqual(comIn, comFrom) && lesserEqual(comIn, comTo) );
        }// end method greaterEqual

}
