package de.cnc.expression.util;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * day of week enum.
 * @author hkuecker
 */
public class DayOfWeek
implements Comparable {

    /** Sonntag */
    public static final DayOfWeek SUNDAY = new DayOfWeek("SUNDAY" , Calendar.SUNDAY);
    
    /** Montag */
    public static final DayOfWeek MONDAY = new DayOfWeek("MONDAY" , Calendar.MONDAY);
    
    /** Dienstag */
    public static final DayOfWeek TUESDAY = new DayOfWeek("TUESDAY" , Calendar.TUESDAY);
    
    /** Mittwoch */
    public static final DayOfWeek WEDNESDAY = new DayOfWeek("WEDNESDAY" , Calendar.WEDNESDAY);
    
    /** Donnerstag */
    public static final DayOfWeek THURSDAY = new DayOfWeek("THURSDAY" , Calendar.THURSDAY);
    
    /** Freitag */
    public static final DayOfWeek FRIDAY = new DayOfWeek("FRIDAY" , Calendar.FRIDAY);
    
    /** Samstag */
    public static final DayOfWeek SATURDAY = new DayOfWeek("SATURDAY" , Calendar.SATURDAY);
    
    
    /** Name */
    private String name;
    
    /**
     * numerischer Code
     * Sonntag (1) bis Samstag (7)
     * wie in {@link Calendar}.
     */
    private int value;
    
    /**
     * @return Returns the name.
     */
    public String getName() {
        return this.name;
    }
    /**
     * @return Returns the value.
     */
    public int getValue() {
        return this.value;
    }
    /**
     * Konstruktor fr interne Verwendung.
     * 
     * @param pName
     * @param pValue
     */
    private DayOfWeek(//--
            final String pName, //--
            final int pValue) {
        this.name = pName;
        this.value = pValue;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object pObj) {
        if (pObj instanceof DayOfWeek //--
                && ( (DayOfWeek) pObj).value == this.value ) {
            return true;
        }
        return false;
    }

    /* (non-Javadoc)
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object pObj) {
        return this.value - ((DayOfWeek)pObj).value;
    }
    
    /**
     * get day of week for the specified date.
     * @param pDate
     * @return
     */
    public static DayOfWeek getInstance(final Date pDate) {
        final GregorianCalendar cal = new GregorianCalendar();
        cal.setTime( pDate );
        int dayOfWeekInt = cal.get( Calendar.DAY_OF_WEEK );
        DayOfWeek retDayOfWeek = getInstance( dayOfWeekInt );
        return retDayOfWeek;
    }

    /**
     * get day of week for the specified int value.
     * @param pDate
     * @return
     */
    public static DayOfWeek getInstance(final int pInt) {
        switch (pInt) {
        case Calendar.SUNDAY:
            return SUNDAY;
            //break;
        case Calendar.MONDAY:
            return MONDAY;
            //break;
        case Calendar.TUESDAY:
            return TUESDAY;
            //break;
        case Calendar.WEDNESDAY:
            return WEDNESDAY;
            //break;
        case Calendar.THURSDAY:
            return THURSDAY;
            //break;
        case Calendar.FRIDAY:
            return FRIDAY;
            //break;
        case Calendar.SATURDAY:
            return SATURDAY;
            //break;
        default:
            return null;
            //break;
        }
    }
    
    /**
     * get day of week for the specified Number object.
     * @param pDate
     * @return
     */
    public static DayOfWeek getInstance(final Number pNumber) {
        if (pNumber == null){
            return null;
        }
        return getInstance( pNumber.intValue() );
    }

    /**
     * get day of week for the specified object.
     * @param pDate
     * @return
     */
    public static DayOfWeek getInstance(final Object pObject) {
        if (pObject == null) {
            return null;
        }
        if (pObject instanceof DayOfWeek){
            return (DayOfWeek) pObject;
        }
        if  (pObject instanceof Date){
            return getInstance( (Date) pObject );
        }
        if  (pObject instanceof Number){
            return getInstance( (Number) pObject );
        }
        throw new IllegalArgumentException( "" + pObject + " class: " + pObject.getClass().getName() );
    }

    /**
     * Debug.
     */
    public String toString(){
        return this.name;   
    }

}
