package de.cnc.expression.util;

import java.util.ArrayList;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.Expression;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;

/**
 * auf Expression typisierte ArrayList . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: ExpressionArrayList.java,v 1.1 2005/04/12 10:09:29 hkuecker Exp $
 */
public class ExpressionArrayList
{

    /**
     * eigentliche ArrayList
     */
    private final ArrayList exprArrList = new ArrayList();

    /**
     * typisiertes get
     */
    public Expression get(int iPaIndex)
    {
        return (Expression) this.exprArrList.get(iPaIndex);
    } // end method get

    /**
     * typisiertes letztes Element
     */
    public Expression tail()
    {
        return (Expression) this.exprArrList.get(this.exprArrList.size() - 1);
    } // end method tail

    /**
     * typisiertes add
     */
    public boolean add(Expression strPaValue)
    {
        return this.exprArrList.add(strPaValue);
    } // end method add

    /**
     * size
     */
    public int size()
    {
        return this.exprArrList.size();
    } // end method size

    /**
     * @return empty ja oder nein
     */
    public boolean isEmpty()
    {
        return this.exprArrList.isEmpty();
    }

    /**
     * Debug-Ausgabe
     */
    public String toString()
    {
        return "" + this.exprArrList;
    }

    /**
     * Operator-Prioritt organisieren (klammern)
     */
    public final void organizeOperatorPriority()
    {
        for (int i = 0; i < this.exprArrList.size(); i++)
        {
            Expression expr = (Expression) this.exprArrList.get( i );
            expr.organizeOperatorPriority();
        }
    }

    /**
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    public final Object[] getCheckReturnTypes(//--
            final AbstractRuntimeEnvironment pRunEnv)
        throws ExpressionParseException
    {
        Object[] typArr = new Object[]{ Types.UNKNOWN_CLASS };
        for (int i = 0; i < this.exprArrList.size(); i++)
        {
            Expression expr = (Expression) this.exprArrList.get( i );
            typArr = expr.getCheckReturnTypes(pRunEnv);
        }
        return typArr;
    }

    /**
     * Prfen ob es sich um einen konstanten Wert (Literal oder konstanter
     * Ausdruck handelt).
     * 
     * @see de.cnc.expression.AbstractToken#isConst(de.cnc.expression.AbstractRuntimeEnvironment)
     */
    public final boolean isConst(final AbstractRuntimeEnvironment pRunEnv)
            throws ExpressionEvaluationException
    {
        for (int i = 0; i < this.exprArrList.size(); i++)
        {
            AbstractToken element = (AbstractToken) this.exprArrList.get(i);
            if (!element.isConst(pRunEnv))
            {
                return false;
            }
        }
        return true;
    }

} // end class ExpressionArrayList
