package de.cnc.expression.util;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * month enum.
 * @author hkuecker
 */
public class Month
implements Comparable {

    /** Januar */
    public static final Month JANUARY = new Month("JANUARY" , Calendar.JANUARY);
    
    /** Februar */
    public static final Month FEBRUARY = new Month("FEBRUARY" , Calendar.FEBRUARY);
    
    /** Mrz */
    public static final Month MARCH = new Month("MARCH" , Calendar.MARCH);
    
    /** April */
    public static final Month APRIL = new Month("APRIL" , Calendar.APRIL);
    
    /** Mai */
    public static final Month MAY = new Month("MAY" , Calendar.MAY);
    
    /** Juni */
    public static final Month JUNE = new Month("JUNE" , Calendar.JUNE);
    
    /** Juli */
    public static final Month JULY = new Month("JULY" , Calendar.JULY);
    
    /** August */
    public static final Month AUGUST = new Month("AUGUST" , Calendar.AUGUST);
    
    /** September */
    public static final Month SEPTEMBER = new Month("SEPTEMBER" , Calendar.SEPTEMBER);
    
    /** Oktober */
    public static final Month OCTOBER = new Month("OCTOBER" , Calendar.OCTOBER);
    
    /** November */
    public static final Month NOVEMBER = new Month("NOVEMBER" , Calendar.NOVEMBER);
    
    /** Dezember */
    public static final Month DECEMBER = new Month("DECEMBER" , Calendar.DECEMBER);
    
    
    /** Name */
    private String name;
    
    /**
     * numerischer Code
     * Sonntag (1) bis Samstag (7)
     * wie in {@link Calendar}.
     */
    private int value;
    
    /**
     * @return Returns the name.
     */
    public String getName() {
        return this.name;
    }
    /**
     * @return Returns the value.
     */
    public int getValue() {
        return this.value;
    }
    /**
     * Konstruktor fr interne Verwendung.
     * 
     * @param pName
     * @param pValue
     */
    private Month(//--
            final String pName, //--
            final int pValue) {
        this.name = pName;
        this.value = pValue;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object pObj) {
        if (pObj instanceof Month
                && ( (Month) pObj).value == this.value ) {
            return true;
        }
        return false;
    }

    /* (non-Javadoc)
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object pObj) {
        return this.value - ((Month)pObj).value;
    }
    
    /**
     * get day of week for the specified date.
     * @param pDate
     * @return
     */
    public static Month getInstance(final Date pDate) {
        final GregorianCalendar cal = new GregorianCalendar();
        cal.setTime( pDate );
        int dayOfWeekInt = cal.get( Calendar.MONTH );
        Month retDayOfWeek = getInstance( dayOfWeekInt );
        return retDayOfWeek;
    }

    /**
     * get day of week for the specified int value.
     * @param pDate
     * @return
     */
    public static Month getInstance(final int pInt) {
        switch (pInt) {
        case Calendar.JANUARY:
            return JANUARY;
            //break;
        case Calendar.FEBRUARY:
            return FEBRUARY;
            //break;
        case Calendar.MARCH:
            return MARCH;
            //break;
        case Calendar.MAY:
            return MAY;
            //break;
        case Calendar.JUNE:
            return JUNE;
            //break;
        case Calendar.JULY:
            return JULY;
            //break;
        case Calendar.AUGUST:
            return AUGUST;
            //break;
        case Calendar.SEPTEMBER:
            return SEPTEMBER;
            //break;
        case Calendar.OCTOBER:
            return OCTOBER;
            //break;
        case Calendar.NOVEMBER:
            return NOVEMBER;
            //break;
        case Calendar.DECEMBER:
            return DECEMBER;
            //break;
        default:
            return null;
            //break;
        }
    }
    
    /**
     * get month for the specified Number object.
     * @param pDate
     * @return
     */
    public static Month getInstance(final Number pNumber) {
        if (pNumber == null){
            return null;
        }
        return getInstance( pNumber.intValue() );
    }

    /**
     * get month for the specified object.
     * @param pDate
     * @return
     */
    public static Month getInstance(final Object pObject) {
        if (pObject == null) {
            return null;
        }
        if (pObject instanceof Month){
            return (Month) pObject;
        }
        if  (pObject instanceof Date){
            return getInstance( (Date) pObject );
        }
        if  (pObject instanceof Number){
            return getInstance( (Number) pObject );
        }
        throw new IllegalArgumentException( "" + pObject + " class: " + pObject.getClass().getName() );
    }

    /**
     * Debug.
     */
    public String toString(){
    	return this.name;   
    }
}
